<?php
	/*
	* 2018 - VEGA Informatique
	*
	* NOTICE OF LICENSE
	*
	* This source file is subject to the Open Software License (OSL 3.0)
	* that is bundled with this package in the file LICENSE.txt.
	* It is also available through the world-wide-web at this URL:
	* http://opensource.org/licenses/osl-3.0.php
	*
	* DISCLAIMER
	*
	* Do not edit or add to this file if you wish to upgrade PolarisLink to newer
	* versions in the future. 
	*
	*  @author VEGA Informatique <contact@vega-info.fr>
	*  @copyright  2018 VEGA Informatique
	*  @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
	*/

	if (!defined('_PS_VERSION_'))
		exit;	
		
	require_once("PolarisLinkException.php");
	require_once("PolarisLinkSite.php");	
	
	/*
	 * Classe de base pour gérer les appels API, s'occuper de la sécurité, et gérer la distribution
	 * de la documentation
	 */
	abstract class PolarisLinkAPI extends ModuleFrontController
	{
		protected $AuthByTokenAllowed = false;
		
		/* ----------------------------------------------------------
		 * Début des propriétés
		 * ---------------------------------------------------------- */		 
		 protected $Link = null;
		
		/* ----------------------------------------------------------
		 * Méthodes overridable
		 * ---------------------------------------------------------- */		
		
		/* Exécution de l'API */
		protected abstract function runAPI();
				
				
		/* ----------------------------------------------------------
		 * Méthodes internes/publiques
		 * ---------------------------------------------------------- */
		
		public function __construct()
		{
			$context = Context::getContext();
			
			// On connecte avec le premier admin, et si non trouvé, le premier employé qui vient
			
			$employees = Db::getInstance()->executeS('SELECT `id_employee` FROM `'._DB_PREFIX_.'employee` WHERE active = 1 ORDER BY id_profile ASC, id_employee ASC');
			foreach ($employees as $employee)
			{			
				$context->employee = new Employee($employee["id_employee"]);
				break;
			}
			
			$this->content_only = true;
			$this->display_header = false;
			$this->display_footer = false;
			$this->display_header_javascript = true;
			$this->ajax = true;
			
			parent::__construct();
			
			// Chargement de la configuration
			$this->module->LoadConfig();
		}
		
		protected function displayMaintenancePage()
		{
			// Désactivation de la maintenance !
		}
		
		/*
			Génère une exception par rapport à une erreur de base de données
		*/
		public static function exceptionOnDbError($action = NULL)
		{
			$db = Db::getInstance();
			$err = $db->getMsgError();
			if (!empty($err))
			{
				if (empty($action))
					throw new Exception($err);
				throw new Exception(sprintf("Erreur lors de l'action «%s» : %s", $action, $err));
			}
		}
		
		/*
			Traite une erreur
		*/
		protected function logError($ex)
		{
			if (get_class($ex) == "PolarisLinkAPI_IMPORT_Exception")
			{
				$error = $ex->getPrevious()->getMessage()." lors du traitement du segment : ".htmlentities(str_replace("\n", " ", $ex->data_on_error->asXML())."");
			}
			else if (get_class($ex) == "Exception")
			{
				$error = $ex->getMessage();
			}
			else
				$error = $ex;
							
			if (isset($_REQUEST["debug"]) && $_REQUEST["debug"] > 0)
				echo $error."<br/>";
				
			Logger::addLog("Erreur : " . $error, 2, 1, "PolarisLink", $this->Link->LinkID, true);
		}

				
		public function initCursedPage()
		{
			http_response_code(403);
			$this->layout = "access-denied.tpl";			
		}
				
		public function checkAccess()
		{
			$key = "";
			
			if (isset($_REQUEST['ppl_key'])) 
			{
				$key = trim($_REQUEST['ppl_key']);
			}
			else 
			{
				//set http auth headers for apache+php-cgi work around
				if (isset($_SERVER['HTTP_AUTHORIZATION']) && preg_match('/Basic\s+(.*)$/i', $_SERVER['HTTP_AUTHORIZATION'], $matches)) {
					list($name, $password) = explode(':', base64_decode($matches[1]));
					$_SERVER['PHP_AUTH_USER'] = strip_tags($name);
					$_SERVER['PHP_AUTH_PW'] = strip_tags($password);
				}

				//set http auth headers for apache+php-cgi work around if variable gets renamed by apache
				if (isset($_SERVER['REDIRECT_HTTP_AUTHORIZATION']) && preg_match('/Basic\s+(.*)$/i', $_SERVER['REDIRECT_HTTP_AUTHORIZATION'], $matches)) {
					list($name, $password) = explode(':', base64_decode($matches[1]));
					$_SERVER['PHP_AUTH_USER'] = strip_tags($name);
					$_SERVER['PHP_AUTH_PW'] = strip_tags($password);
				}

				if (isset($_SERVER['PHP_AUTH_PW'])) 
					$key = trim($_SERVER['PHP_AUTH_PW']);
				
				if (empty($key) && isset($_SERVER['PHP_AUTH_USER']))
					$key = trim($_SERVER['PHP_AUTH_USER']);				
			}

			$this->Link = null;
			if (!empty($key))
			{
				// Contrôle clé d'accès
				foreach ($this->module->Config as $cfg)
				{
					if ($cfg->AccessActive && $cfg->AccessKey == $key)
					{
						// clé trouvé, accès accepté avec cette configuration
						$this->Link = $cfg;
						break;
					}
				}
			}
			
			// Par token ?
			if ($this->AuthByTokenAllowed && $this->module->getAuthTokenStr() == Tools::getValue('pl_auth_tk'))
				return true;
			
			if ($this->Link == null)
			{				
				http_response_code(401);
				//header($_SERVER['SERVER_PROTOCOL'].' 401 Unauthorized');
				header('WWW-Authenticate: Basic realm="This area is restricted. Please provide keys to access."');
				
				$error = new PolarisLinkError();
				$error->error = 401;
				$error->msg = 'unauthorized';				
				$this->answerJSON($error);
				
				die();
			}
									
			return true;
		}
		
		public function apiCheckAccess()
		{
			return true;
		}
		
		public function answerJSON($object)
		{
			header("Content-Type: application/json; charset=utf-8");
			echo json_encode($object);
		}
		
		public static function getRelativePath($from, $to)
		{
			// some compatibility fixes for Windows paths
			$from = is_dir($from) ? rtrim($from, '\/') . '/' : $from;
			$to   = is_dir($to)   ? rtrim($to, '\/') . '/'   : $to;
			$from = str_replace('\\', '/', $from);
			$to   = str_replace('\\', '/', $to);

			$from     = explode('/', $from);
			$to       = explode('/', $to);
			$relPath  = $to;

			foreach($from as $depth => $dir) {
				// find first non-matching dir
				if($dir === $to[$depth]) {
					// ignore this directory
					array_shift($relPath);
				} else {
					// get number of remaining dirs to $from
					$remaining = count($from) - $depth;
					if($remaining > 1) {
						// add traversals up to first matching dir
						$padLength = (count($relPath) + $remaining - 1) * -1;
						$relPath = array_pad($relPath, $padLength, '..');
						break;
					} else {
						$relPath[0] = './' . $relPath[0];
					}
				}
			}
			return implode('/', $relPath);
		}
				
		public static function GetPolarisPhotoPathFor($id_link, $id_polaris, $short = false)
		{
			$path = ($short) ? "" : _PS_IMG_DIR_;
			
			$path .= "polarislink/lk-".$id_link."/";
			
			$str = strval(intval($id_polaris));
			for ($i = 0; $i < strlen($str); $i++)
				$path .= $str[$i]."/";

			if (!file_exists($path))
				mkdir($path, 0755, true);
								
			$path .= $id_polaris.".jpg";
			return $path;			
		}
				
		public function initContent()
		{
			try
			{
				parent::initContent();
				
				/* contrôle de l'appel */			
			
				/* on vérifie plus en profondeur les autorisations */
				if (!$this->apiCheckAccess())
					throw new PolarisLinkException("accès interdit", 403);

				/* tout est bon, éxécution */
				$this->runAPI();
			}
			catch (PolarisLinkException $ex)
			{
				http_response_code($ex->HttpErrorCode);
				$this->answerJSON($ex->getError());
			}
			catch (Exception $ex)
			{
				$error = new PolarisLinkError();
				$code = $ex->getCode();
				$error->error = $code > 10000 ? $code - 10000 : 422;
				$error->msg = $ex->getMessage();
				$this->answerJSON($error);
			}
		}
		
		public function display()
		{
		}
	} 
