<?php
	/*
	* 2018 - VEGA Informatique
	*
	* NOTICE OF LICENSE
	*
	* This source file is subject to the Open Software License (OSL 3.0)
	* that is bundled with this package in the file LICENSE.txt.
	* It is also available through the world-wide-web at this URL:
	* http://opensource.org/licenses/osl-3.0.php
	*
	* DISCLAIMER
	*
	* Do not edit or add to this file if you wish to upgrade PolarisLink to newer
	* versions in the future. 
	*
	*  @author VEGA Informatique <contact@vega-info.fr>
	*  @copyright  2018 VEGA Informatique
	*  @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
	*/

	require_once("PolarisLinkSite.php");	
	require_once("PolarisLinkCacheManager.php");		
	
	abstract class PolarisLinkImportBase 
	{
		public $DebugMode = false;
		protected $JobManager = null;
		protected $Link = null;
		protected $DB = null;		
		
		protected function setCache(PolarisLinkCache $cache)
		{
			$this->JobManager->setCache($cache);
		}
		
		protected function getCache($type)
		{
			return $this->JobManager->getCache($type);
		}
		
		/*
			On journalise l'erreur
		*/
		protected function logError($ex)		
		{
			$this->JobManager->logError($ex);
		}		
		
		protected function SetShopContext($opt)
		{
			if ($opt == null || $opt == 0 || !Shop::isFeatureActive())
				return;

			if (strlen($opt) > 0 && $opt[0] == '*')
				Shop::setContext(Shop::CONTEXT_GROUP, substr($opt, 1));
			else
				Shop::setContext(Shop::CONTEXT_SHOP, $opt);			
		}
		
		protected function SetCatalogContext()
		{
			$this->SetShopContext($this->Link->CatalogShopID);
		}
		
		protected function SetPricesContext()
		{
			$this->SetShopContext($this->Link->PricesShopID);
		}
		
		/*
			Création d'un champ multilangage compatible avec Prestashop
		*/		
		protected function createMultiLangField($value = "", $allLanguages = true)
		{
			if (!$allLanguages)
				return array($this->Link->LangID => $value);
			
			$res = array();
			foreach (Language::getIDs(false) as $id_lang) 
				$res[$id_lang] = $value;

			return $res;
		}
		
		/*
			Obtient la valeur de la balise ou de l'attribut, émet une erreur si non spécifiée
		*/
		protected function GetMandatory($data, $xml_node_name, $attribute_name = null)
		{
			if (empty($attribute_name))
			{
				if (!isset($data->$xml_node_name) || (string) $data->$xml_node_name == null || (string) $data->$xml_node_name == "")
					throw new Exception("la balise ".$xml_node_name." n'a pas été spécifiée alors qu'attendue");
				
				return $data->$xml_node_name;
			}
			else
			{
				$value = $data;
				if (!empty($xml_node_name))
					$value = $data->$xml_node_name;
				
				if (!isset($value[$attribute_name]) || empty($value[$attribute_name]))
					throw new Exception("l'attribut ".$xml_node_name."[".$attribute_name."] n'a pas été spécifiée alors qu'attendu");
				
				return $value[$attribute_name];
			}
		}
		
		/*
			Obtient la valeur de la balise ou de l'attribut, renvoi la valeur par défaut si non spécifiée
		*/
		protected function GetOptional($data, $xml_node_name, $attribute_name = null, $defaultValue = null)
		{
			if (empty($attribute_name))
			{
				if (!isset($data->$xml_node_name) || (string) $data->$xml_node_name == null || (string) $data->$xml_node_name == "")
					return $defaultValue;
				return $data->$xml_node_name;
			}
			else
			{
				$value = $data;
				if (!empty($xml_node_name))
					$value = $data->$xml_node_name;
				
				if (!isset($value[$attribute_name]) || empty($value[$attribute_name]))
					return $defaultValue;
				
				return $value[$attribute_name];
			}
		}

		/*
			Change la propriété de l'objet et renvoi vrai si qqchose à changé
		*/
		protected function change(&$obj, $propertyName, $newValue)
		{
			if ($obj->$propertyName != $newValue)
			{
				$obj->$propertyName = $newValue;
				return true;
			}
			return false;
		}
		
		protected function getLgProperty($value, $mandatory = false)
		{
			$langs = array();
			
			if ($mandatory)
			{
				foreach (Language::getLanguages(false) as $lang)
					$langs[$lang["id_lang"]] = "";
			}
						
			$done = 0;
			if ($value != null && count($value) > 0)
			{
				foreach ($value->children() as $langXmlId => $child) 
				{		
					$lang = $this->JobManager->getLangFor($langXmlId);					
					if ($lang != NULL)					
						$langs[$lang["id_lang"]] = (string) $child;
					$done++;
				}
			}
			if ($done == 0)
			{
				return $this->createMultiLangField((string) $value, $mandatory);
			}
			else
				return $langs;
		}
		
		protected function AutoFillEmptyLangValue($field, $defaultValue = null)
		{
			$fillValue = NULL;
			$hasEmpty = false;
			foreach ($field as $langId => $value)
			{
				if (empty($value))
					$hasEmpty = true;
				else if (empty($fillValue))
					$fillValue = $value;
			}
			
			if ($hasEmpty)
			{
				if (empty($fillValue))
					$fillValue = $defaultValue;
					
				foreach ($field as $langId => $value)
				{
					if (empty($value))
						$field[$langId] = $fillValue;
				}
			}
			
			return $field;
		}
				
		/*
			Copie une propriété présente dans $value pour la pousser
			dans l'instance Prestashop $obj.
			Si pas trouvé, utilise $default pour remplir la valeur			
		*/		
		protected function updateLgProperty($obj, $property, $value, $default = NULL, $defaultAll = false, $checkFunc = null)
		{
			$done = 0;

			// A priori, on communique avec des balises de langues
			if ($value != null && count($value) > 0)
			{
				if ($defaultAll)
					$obj->$property = $this->createMultiLangField($default);
				
				foreach ($value->children() as $langXmlId => $child) 
				{		
					$lang = $this->JobManager->getLangFor($langXmlId);					
					if ($lang != NULL)
						$this->updateLgPropertyWithLangId($lang["id_lang"], $obj, $property, $child, $default, $checkFunc);
					$done++;
				}
			}
			
			// Pas de descriptif de langue ? langue par défaut
			if ($done == 0)
				$this->updateLgPropertyWithLangId($this->Link->LangID, $obj, $property, $value, $default, $checkFunc);
		}			
		
		/*
			Copie une propriété présente dans $value pour la pousser
			dans l'instance Prestashop $obj.
			Si pas trouvé, utilise $default pour remplir la valeur			
		*/		
		protected function updateLgPropertyWithLangId($langId, $obj, $property, $value, $default = NULL, $checkFunc = null)
		{			
			// Balise positionnée, nous modifions !
			$value = (string) $value;
			if (empty($value))
				$value = $default;		
			
			// On évite d'écraser si la balise est vide
			if (is_null($value))
				return;

			if ($checkFunc != null)
				$value = $checkFunc($value);
			
			$p = $obj->$property;
			if (!isset($p[$langId]) || $p[$langId] != $value)
			{
				$p[$langId] = $value;
				$obj->$property = $p;
			}			
		}
		
		/*
			Copie une propriété présente dans $value pour la pousser
			dans l'instance Prestashop $obj.
			Si pas trouvé, utilise $default pour remplir la valeur			
		*/		
		protected function updateProperty($obj, $property, $value, $default = NULL, $checkFunc = null)
		{			
			$value = (string) $value;
			if (empty($value))
				$value = $default;
			
			// On évite d'écraser si la balise est vide
			if (is_null($value))
				return;
			
			if ($checkFunc != null)
				$value = $checkFunc($value);
			
			if ($obj->$property != $value)
				$obj->$property = $value;
		}		
		
		protected function GetDateTime($value)
		{
			return $value;
		}		
		
		/*
			Renvoi le premier argument non vide
		*/
		protected function FirstNonEmpty($arg1, $arg2 = null, $arg3 = null, $arg4 = null, $arg5 = null, $arg6 = null)
		{
			if (!empty($arg1) && !empty(trim($arg1))) return $arg1;
			if (!empty($arg2) && !empty(trim($arg2))) return $arg2;
			if (!empty($arg3) && !empty(trim($arg3))) return $arg3;
			if (!empty($arg4) && !empty(trim($arg4))) return $arg4;
			if (!empty($arg5) && !empty(trim($arg5))) return $arg5;
			if (!empty($arg6) && !empty(trim($arg6))) return $arg6;
			
			return null;
		}
		
		/*
			Renvoi le nombre de combinaison d'un produit, ou 0 si ce dernier n'existe pas
		*/
		protected function CountProductNbCombinations($product_id)
		{
			$sql = "SELECT COUNT(id_product_attribute) AS c
					FROM " . _DB_PREFIX_ . "product_attribute
					WHERE id_product = ".$product_id." AND id_product_attribute <> 0";
					
			if ($rows = $this->DB->ExecuteS($sql, true, false)) 
			{
				foreach ($rows as $row)
					return $row["c"];
				return 0;
			}
			
			return NULL;
		}
		
		protected function GetProduct($id_product)
		{
			if (empty($id_product))
				return NULL;
			
			if (is_array($id_product))
				$id_product = $id_product["id_product"];
			
			$product = new Product($id_product);
			if ((int) $product->id == (int) $id_product)
				return $product;
			return NULL;
		}		
		
		protected function GetFirstReferencesFromList($list)
		{
			return $this->GetReferencesFromList($list, true);
		}
		
		protected function GetReferencesFromList($list, $onlyFirst = false)
		{
			if (empty($list))
				return $onlyFirst ? NULL : array();
			
			$ids = array();
			foreach (explode(',', $list) as $e)
			{
				$id = trim($e);
				if (!empty($id))
				{
					if ($onlyFirst)
						return $id;
					$ids[] = $id;
				}
			}
			
			if ($onlyFirst)
				return count($ids) > 0 ? $ids[0] : NULL;

			return $ids;
		}
		
		/*
			Recherche un produit et sa combinaison prestashop correspondant à la référence Polaris, ou au code barre gencode, ou 
			null si non trouvé et pas d'exception demandée.
			$laszyProduct = si vrai, ne s'assure pas que le produit trouvé est dépourvu de combinaisons si le code trouvé est celui du produit et non d'une combinaison
		*/
		public function GetProductCombinationFor($referencesList, 
							$eansListe = NULL, 
							$exceptionIfNotFound = false,
							$searchInPolarisBackup = false,
							$lazyProduct = false
							)
		{
			$suppliers = $this->Link->SupplierID;
			if ($searchInPolarisBackup)
				$suppliers .= ", ".$this->Link->SupplierBackupID;
			
			$references = array();
			foreach (explode(',', $referencesList) as $reference)
			{
				$elt = trim($reference);
				if (!empty($reference))
					$references[] = "'".pSQL($reference)."'";
			}
			
			$eans = array();
			foreach (explode(',', $eansListe) as $ean)
			{
				$elt = trim($ean);
				if (!empty($ean))
					$eans[] = "'".pSQL($ean)."'";
			}			
			
			// On recherche le produit
			if (count($references) > 0)
			{
				$sql = "SELECT s.id_product, s.id_product_attribute
						FROM " . _DB_PREFIX_ . "product_supplier AS s
						LEFT JOIN " . _DB_PREFIX_ . "product AS p ON (p.id_product = s.id_product)
						LEFT JOIN " . _DB_PREFIX_ . "product_attribute AS pa ON (pa.id_product = s.id_product AND pa.id_product_attribute = s.id_product_attribute)
						WHERE s.id_supplier IN (".$suppliers.") AND s.product_supplier_reference IN (".implode(',', $references).") AND s.id_product_attribute <> 0
							AND p.id_product IS NOT NULL
							AND pa.id_product_attribute IS NOT NULL
						ORDER BY s.id_product ASC, s.id_product_attribute ASC";
				if ($rows = $this->DB->ExecuteS($sql, true, false)) 
				{
					foreach ($rows as $row)
						return $row;
				}
			}
			
			if (count($eans) > 0)
			{
				// Non trouvé ? On essaye avec le code EAN (gencode) !
				$sql = "SELECT pa.id_product, pa.id_product_attribute
						FROM " . _DB_PREFIX_ . "product_attribute AS pa
							LEFT JOIN " . _DB_PREFIX_ . "product AS p ON (p.id_product = pa.id_product)
						WHERE pa.ean13 IN (".implode(',', $eans).") 
							AND p.id_product IS NOT NULL
						ORDER BY pa.id_product ASC, pa.id_product_attribute ASC";
				if ($rows = $this->DB->ExecuteS($sql, true, false)) 
				{
					foreach ($rows as $row)
						return $row;
				}
			}
			
			if (count($references) > 0)
			{
				// Ok... Peut-être que c'est un produit SANS combinaison,
				// du coup, on importe que si le produit est SANS combinaison			
				$sql = "SELECT s.id_product, 0 AS id_product_attribute
						FROM " . _DB_PREFIX_ . "product_supplier AS s
						LEFT JOIN " . _DB_PREFIX_ . "product AS p ON (p.id_product = s.id_product)
						WHERE s.id_supplier IN (".$suppliers.") AND product_supplier_reference IN (".implode(',', $references).") 
							AND p.id_product IS NOT NULL
						ORDER BY id_product ASC";
						
				if ($rows = $this->DB->ExecuteS($sql, true, false)) 
				{
					foreach ($rows as $row)
					{
						if ($lazyProduct || $this->CountProductNbCombinations($row["id_product"]) == 0)
							return $row;
					}
				}
			}
			
			if (count($eans) > 0)
			{
				// Non trouvé ? On essaye avec le code EAN (gencode) sans combinaison !
				$sql = "SELECT p.id_product, 0 AS id_product_attribute
						FROM " . _DB_PREFIX_ . "product AS p							
						WHERE p.ean13 IN (".implode(',', $eans).") 
						ORDER BY p.id_product ASC";
				if ($rows = $this->DB->ExecuteS($sql, true, false)) 
				{
					foreach ($rows as $row)
						if ($lazyProduct || $this->CountProductNbCombinations($row["id_product"]) == 0)
							return $row;
				}
			}
			
			// Non trouvé ?
			if ($exceptionIfNotFound)
			{
				if (!empty($referencesList) && !empty($eansListe))
					throw new Exception(sprintf("produit ref %s ou ean %s non trouvé", $referencesList, $ean));
				if (!empty($referencesList))
					throw new Exception(sprintf("produit ref %s non trouvé", $referencesList));
				if (!empty($eansListe))
					throw new Exception(sprintf("produit ean %s non trouvé", $eansListe));
				throw new Exception(sprintf("produit non trouvé"));
			}
			
			return NULL;
		}
		
		/*
			Renvoi la photo Polaris pour une photo du répertoire
			si la photo du répertoire n'est pas une photo Polaris, renvoi -1
			sinon renvoi NULL si aucune photo
		*/
		protected function GetPolarisPhotoFor($filename)
		{
			if (!file_exists($filename))
				return NULL;
			
			if (!is_link($filename))
				return -1;
			
			// Lecture de la destination
			return basename(readlink($filename), ".jpg");
		}
		
		/*
		* Reconstruction des miniatures
		*/
		protected function RebuildThumbs($thumbTypes, $mini, $path, $id)
		{
			$res = true;
			$generate_hight_dpi_images = (bool)Configuration::get('PS_HIGHT_DPI');

			/* Generate image with differents size */
			foreach (ImageType::getImagesTypes($thumbTypes) as $k => $image_type) 
			{
				$res &= ImageManager::resize(
					$path.$id.'.jpg',
					$path.$id.'-'.stripslashes($image_type['name']).'.jpg',
					(int)$image_type['width'],
					(int)$image_type['height']
				);

				if ($generate_hight_dpi_images) {
					$res &= ImageManager::resize(
						$path.$id.'.jpg',
						$path.$id.'-'.stripslashes($image_type['name']).'2x.jpg',
						(int)$image_type['width']*2,
						(int)$image_type['height']*2
					);
				}
			}

			$current_logo_file = _PS_TMP_IMG_DIR_.$mini.'_mini_'.$id.'_'.Context::getContext()->shop->id.'.jpg';

			if ($res && file_exists($current_logo_file)) {
				unlink($current_logo_file);
			}
			
			return $res;
		}
		
		/*
			Importe un logo 
		*/
		protected function ImportPhotoLogo($hint, $photosPath, $title, $itemName, $psObj, $listPhotoPolaris)
		{
			if ($photo = $this->GetReferencesFromList($listPhotoPolaris, true))
			{
				// On lit la photo actuelle
				$filename_final = $photosPath.($psObj->id).".jpg";
				if ($this->GetPolarisPhotoFor($filename_final) != $photo)
				{
					// L'image a changé
					$psObj->deleteImage();
					
					if ($photo > 0)
					{
						// On pousse la nouvelle sous forme de lien
						$photo_polaris_filename = PolarisLinkAPI::GetPolarisPhotoPathFor($this->Link->LinkID, $photo);
					
						if (file_exists($photo_polaris_filename))
						{
							symlink(PolarisLinkAPI::getRelativePath($photosPath, $photo_polaris_filename), $filename_final);
							$this->RebuildThumbs($title, $itemName, $photosPath, $psObj->id);
						}
						else
							$this->logError("photo Polaris n°".$photo." pour la ".$hint." #".$psObj->id." introuvable sur le disque");
					}
				}				
			}
		}
		
		protected function getPSObjHash($psObj)
		{
			return sha1(serialize($psObj->getFields()).serialize($psObj->getFieldsLang()).serialize($psObj->getFieldsShop()));
		}
		
		/*
			Initialisation, on se note un certain nombre de paramètres
		*/
		public function __construct()
		{			
		}
		
		/*
			Initialise l'instance
		*/		
		public function Init(PolarisLinkJobManager $jobManager, PolarisLinkSite $link)
		{
			$this->DB = Db::getInstance();
			$this->Link = $link;
			$this->JobManager = $jobManager;
			$this->CacheManager = new PolarisLinkCacheManager();
			$this->OnInit();
		}
		
		/*
			Méthode appelée à l'initialisation
		*/
		protected function OnInit()
		{
		}
	
		/*
			Méthode appelé sur un élément à importer
		*/
		protected function OnImportData($data)
		{
		}

		public function buildHashValues($objectModel)
		{
			if ($objectModel == NULL)
				return "";
			
			return sha1(json_encode(array(
				$objectModel->getFields(),
				$objectModel->getFieldsShop(),
				$objectModel->getFieldsLang(),
			)));			
		}

		/*
			Obtient la référence d'un XML
		*/
		protected function getReferenceFromData($data)
		{
			$ref = null;
			
			if (empty($ref) && isset($data["ref"])) $ref = $data["ref"];
			if (empty($ref) && isset($data["sku"])) $ref = $data["sku"];
			if (empty($ref)) $ref = $data->reference;

			return $ref;
		}

		/*
			Méthode appelé pour importer un élément
		*/
		public function import(XMLReader $docxml)
		{
			try
			{
				$strData = $docxml->readOuterXML();
				if ($this->DebugMode)
				{
					echo "Import : ";
					try
					{
						$data = new SimpleXMLElement($strData);
						
						$doc = new DomDocument();
						$doc->loadXML($data->asXML());
						$doc->preserveWhiteSpace = false;
						$doc->formatOutput = true;

						echo $doc->saveXML();						
					}
					catch (Exception $ex)
					{
						var_dump($strData);
					}
					echo "\n";
				}
				else
					$data = new SimpleXMLElement($strData);
				
				$this->OnImportData($data);			
			}
			catch (Exception $ex)
			{
				$ref = "";
				if (!empty($data))
					$ref = $this->getReferenceFromData($data);

				if (empty($ref))
					$ref = "?";
				
				throw new Exception(sprintf("%s, ref. %s", $docxml->name, $ref), 0, $ex);
			}
		}
	}
	