<?php
	/*
	* 2018 - VEGA Informatique
	*
	* NOTICE OF LICENSE
	*
	* This source file is subject to the Open Software License (OSL 3.0)
	* that is bundled with this package in the file LICENSE.txt.
	* It is also available through the world-wide-web at this URL:
	* http://opensource.org/licenses/osl-3.0.php
	*
	* DISCLAIMER
	*
	* Do not edit or add to this file if you wish to upgrade PolarisLink to newer
	* versions in the future. 
	*
	*  @author VEGA Informatique <contact@vega-info.fr>
	*  @copyright  2018 VEGA Informatique
	*  @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
	*/

	require_once(_PS_ROOT_DIR_ . "/modules/polarislink/classes/PolarisLinkImportBase.php");
	
	/*
	 * ex. : 
	 <customer mail="100007936@test.fr">
        <name>DUPONT</name>
        <surname>Marcel</surname>
        <phone />
        <mobile />
        <memo />
        <date_add>2009-06-19</date_add>
    </customer>
	 */
	class PolarisLinkImport_customer extends PolarisLinkImportBase 
	{
		private $Genders = null;
		
		public function OnInit()
		{
			parent::OnInit();						
		}		

		/*
			Obtient la référence d'un XML
		*/
		protected function getReferenceFromData($data)
		{
			if (isset($data["mail"]))
				return $data["mail"];

			return null;
		}
		
		private function GetIdent($title)
		{
			if ($title == null || trim($title) == "")
				return 0;
			
			$title = trim($title);
			
			if ($this->Genders == null)
			{
				$this->Genders = array();
				foreach (Gender::getGenders() as $gender)
					$this->Genders[$gender->id] = $gender->name;
			}
			
			$ctitle = mb_strtolower($title);
			
			foreach ($this->Genders as $id => $gender)
			{
				$cgender = mb_strtolower($gender);
				if ($cgender == $ctitle || $cgender."." == $ctitle || $cgender == $ctitle.".")
					return $id;
			}
			
			// Toujours là, il faut le créer
			$gender = new Gender();
			$gender->name = $this->createMultiLangField($title);
			
			$gender->type = 0;
			$gender->save();
			$this->Genders = null;
			
			return $gender->id();
		}		
		
		private function RegisterCustomerMail($customer)
		{
			if (empty($customer->id))
				return;
			
			/* Get mail list for a customer ...
			foreach (Db::getInstance()->executeS(sprintf("SELECT CAST(AES_DECRYPT(UNHEX(mail), sha1('%s')) AS char) AS mail FROM "._DB_PREFIX_."polarislink_mails", sha1(_COOKIE_KEY_))) as $row)
				var_dump($row);
			*/
			
			foreach (Db::getInstance()->executeS(
				sprintf("SELECT id_mail FROM "._DB_PREFIX_."polarislink_mails WHERE id_customer = '%s' AND mail = HEX(AES_ENCRYPT('%s', sha1('%s'))) AND mail_len = '%s'",
							$customer->id, $customer->email, sha1(_COOKIE_KEY_), strlen($customer->email))) as $item)
				return;	// Existe déjà
			
			// Si on arrive ici, alors on enregistre l'association
			Db::getInstance()->query(
									sprintf("INSERT INTO "._DB_PREFIX_."polarislink_mails (id_customer, mail, mail_len) VALUES ('%s', HEX(AES_ENCRYPT('%s', sha1('%s'))), '%s')", 
										$customer->id, $customer->email, sha1(_COOKIE_KEY_), strlen($customer->email)));			
		}
		
		/*
			Trouve un client avec son adresse mail
		*/
		public static function GetCustomerForMail($mail, $throwExceptionIfNotFound = false)
		{	
			$id = self::GetCustomerIdForMail($mail, $throwExceptionIfNotFound);
			if (!empty($id))
				return new Customer($id);
			
			return NULL;
		}
		
		/*
			Trouve un numéro de client avec son adresse mail
		*/
		public static function GetCustomerIdForMail($mail, $throwExceptionIfNotFound = false)
		{
			$result = Db::getInstance()->getRow('SELECT id_customer FROM `'._DB_PREFIX_.'customer` WHERE `email` = \''.pSQL($mail).'\'');
			if ($result)
				return $result['id_customer'];
			
			// Pas trouvé, on cherche dans la table des mails			
			foreach (Db::getInstance()->executeS(
										sprintf("SELECT id_customer FROM "._DB_PREFIX_."polarislink_mails WHERE mail = HEX(AES_ENCRYPT('%s', sha1('%s'))) AND mail_len = '%s'",
												$mail, sha1(_COOKIE_KEY_), strlen($mail)), true, false) as $item)
							{
								return $item["id_customer"];
							}							
			
			// arrivé jusqu'ici ? c'est que nous n'avons pas trouvé le client !
			if ($throwExceptionIfNotFound)
				throw new Exception(sprintf("impossible de trouver le client correspondant à l'adresse mail '%s'", $mail));
						
			return NULL;
		}
		
		protected function OnImportData($data)
		{
			if (!$this->Link->OPT_ALLOW_I_CUSTOMERS)
				return;	// Pas d'importation de client !
			
			// Avons nous la référence ?
			$mail = (string) $data["mail"];
			if (empty($mail))
				throw new Exception("aucun mail précisé pour le client");
			
			if (!Validate::isEmail($mail))
				throw new Exception(sprintf("%s n'est pas une adresse valide, le client ne sera pas importé", $mail));
						
			// On localise le client
			$customer = self::GetCustomerForMail($mail);
			
			// Ok, si le client existe, il a été localisé
			$deleted = (int) $data["deleted"];
			if ($deleted)
			{
				// On doit supprimer le client
				if (!empty($customer))
					$customer->delete();
			}
			else
			{
				if (empty($customer))
				{
					$customer = new Customer();
					$customer->show_public_prices = 1;
					$customer->deleted = 0;
					$customer->is_guest = 0;
					$customer->passwd = md5(openssl_random_pseudo_bytes(64));					
					$this->updateProperty($customer, "email", $mail);	// On ne change jamais le mail sur le site, uniquement sur Polaris
					$this->updateProperty($customer, "lastname", $data->lastname, null, 'checkCustomerLastName');	
					$this->updateProperty($customer, "firstname", $data->firstname, null, 'checkCustomerFirstname');
					
					if (empty($customer->lastname))
						$customer->lastname = "INCONNU";
					if (empty($customer->firstname))
						$customer->firstname = "Inconnu";
					
					$this->updateProperty($customer, "date_add", $data->date_add);
					
					$customer->save();
					$customer->addGroups(array($this->Link->CustomersGroupID));					
					$origin = "";
				}				
				else
					$origin = $this->getPSObjHash($customer);
			
				$this->updateProperty($customer, "lastname", $data->lastname, null, 'checkCustomerLastName');	
				$this->updateProperty($customer, "firstname", $data->firstname, null, 'checkCustomerFirstname');
				
				if (empty($customer->lastname))
						$customer->lastname = "INCONNU";
					if (empty($customer->firstname))
						$customer->firstname = "Inconnu";

				if (count($data->title) > 0)
					$customer->id_gender = $this->GetIdent($data->title);
				
				$this->updateProperty($customer, "birthday", $data->birthdate);	
				$this->updateProperty($customer, "optin", empty($data->optin) ? "0" : "1");					
				
				if (!empty($data->memo))
					$this->updateProperty($customer, "note", $data->memo);
				
				$customer->active = 1;
								
				// On sauve les modifications
				if ($this->getPSObjHash($customer) != $origin)
					$customer->save();
				
				$this->RegisterCustomerMail($customer);
			}
		}
	}
	