<?php
	/*
	* 2018 - VEGA Informatique
	*
	* NOTICE OF LICENSE
	*
	* This source file is subject to the Open Software License (OSL 3.0)
	* that is bundled with this package in the file LICENSE.txt.
	* It is also available through the world-wide-web at this URL:
	* http://opensource.org/licenses/osl-3.0.php
	*
	* DISCLAIMER
	*
	* Do not edit or add to this file if you wish to upgrade PolarisLink to newer
	* versions in the future. 
	*
	*  @author VEGA Informatique <contact@vega-info.fr>
	*  @copyright  2018 VEGA Informatique
	*  @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
	*/

	require_once(_PS_ROOT_DIR_ . "/modules/polarislink/classes/PolarisLinkImportBase.php");
	
	/*
	 * ex. : <grouptag ref="yy">
				<name>name</name>
			 </grouptag>
	 */
	class PolarisLinkImport_grouptag extends PolarisLinkImportBase 
	{	
		private static $OwnedGroups = null;
		
		protected function OnInit()
		{
			parent::OnInit();
						
			if ($this->Link->MethodTags == 2)
				$this->setCache(new PolarisLinkExtRefCache($this->Link->LinkID, "grtag_features"));			
		}
		
		public function GetOwnedTagGroups()
		{
			return $this->getCache("pr_grtag_features")->listValues();
		}
		
		private function getFeatureIdByName($names)
		{
			foreach ($names as $id_lang => $name)
			{
				$features = Feature::getFeatures($id_lang, true);
				foreach ($features as $value)
					if ($value["name"] == $name)
						return $value["id_feature"];
			}
		}
		
		public function getOrCreateGroupTag($polarisRef, $names)
		{
			switch ($this->Link->MethodTags)
			{
				case 0:
				case 1:
				case 3:
					// Non géré et par mot clés, on ne passe jamais par ici
					return;
			}			
			
			$cache = $this->getCache("pr_grtag_features");
			
			$psId = $cache->get($polarisRef);
			if (empty($psId))
			{
				
				// On cherche la catégorie par son nom ...
				$psId = $this->getFeatureIdByName($names);
				
				if (empty($psId))
				{
					$obj = new Feature();					
					$obj->name = $this->AutoFillEmptyLangValue($names, $polarisRef);
					$obj->save();
					
					$psId = $obj->id;					
				}
				
				$cache->set($polarisRef, $psId);
			}
			
			return $psId;
		}		
		
		public function ImportData($data)
		{
			if (!$this->Link->OPT_ALLOW_I_CATALOG)
				return;	// Rien à faire ici dans ce cas là !
			
			switch ($this->Link->MethodTags)
			{
				case 0:	// Non géré
				case 3: // Géré par mots clé, ça ne se fait pas ici
					return;
				case 1:	// Géré par catégorie, on reroute
					return $this->JobManager->getImporter("category")->ImportData($data, NULL, "grtag", NULL);
			}
			
			// Avons nous la référence Polaris ?
			$polarisRef = (string) $this->GetMandatory($data, "", "ref");

			// Localisation du tag
			$psId = $this->getCache("pr_grtag_features")->get($polarisRef);
			
			// Activé ?
			$deleted = (string) $this->GetOptional($data, "", "deleted", "0");
						
			if (!$deleted)
			{
				$obj = new Feature($this->getOrCreateGroupTag($polarisRef, $this->getLgProperty($data->name, true)));
				$origin = $this->getPSObjHash($obj);
				$this->updateLgProperty($obj, "name", $data->name);
				
				if ($this->getPSObjHash($obj) != $origin)
					$obj->save();
				
				return $obj->id;
			}
			else if (!empty($psId))
			{		
				// Ce groupe tag n'existe plus, on supprime
				$obj = new Feature($psId);
				$obj->delete();
			}
		}
		
		protected function OnImportData($data)
		{
			$this->ImportData($data);
		}
	}
	