<?php
	/*
	* 2018 - VEGA Informatique
	*
	* NOTICE OF LICENSE
	*
	* This source file is subject to the Open Software License (OSL 3.0)
	* that is bundled with this package in the file LICENSE.txt.
	* It is also available through the world-wide-web at this URL:
	* http://opensource.org/licenses/osl-3.0.php
	*
	* DISCLAIMER
	*
	* Do not edit or add to this file if you wish to upgrade PolarisLink to newer
	* versions in the future. 
	*
	*  @author VEGA Informatique <contact@vega-info.fr>
	*  @copyright  2018 VEGA Informatique
	*  @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
	*/

	require_once(_PS_ROOT_DIR_ . "/modules/polarislink/classes/PolarisLinkImportBase.php");
	
	/*
	 * ex. : <manufacturer ref="pick ouic"><description/></manufacturer>
	 */
	class PolarisLinkImport_manufacturer extends PolarisLinkImportBase 
	{
		public function OnInit()
		{
			parent::OnInit();
			
			$this->setCache(new PolarisLinkExtRefCache($this->Link->LinkID, "manufacturer"));
		}
		
		public function getManufacturerId($polarisRef, $name, $active = true)
		{
			return $this->getCache("pr_manufacturer")->get($polarisRef);
		}	
		
		public function getOrCreateManufacturerId($polarisRef, $name, $active = true)
		{
			$psId = $this->getCache("pr_manufacturer")->get($polarisRef);

			if (empty($psId))
			{
				// On cherche la marque par son nom ...
				$psId = Manufacturer::getIdByName($name);
				
				if (empty($psId))
				{
					$obj = new Manufacturer();
					$obj->name = empty($name) ? $polarisRef : $name;
					$obj->active = $active;
					$obj->save();
					
					$psId = $obj->id;
				}
				
				$this->getCache("pr_manufacturer")->set($polarisRef, $psId);
			}
			
			return $psId;
		}	
		
		protected function OnImportData($data)
		{
			if (!$this->Link->OPT_ALLOW_I_CATALOG)
				return;	// Rien à faire ici dans ce cas là !
			
			// Avons nous la référence Polaris ?
			$polarisRef = (string) $this->GetMandatory($data, "", "ref");

			// Localisation de la marque
			$psId = $this->getCache("pr_manufacturer")->get($polarisRef);
			
			// Activé ?
			$deleted = (string) $this->GetOptional($data, "", "deleted", "0");
			
			if (!$deleted)
			{
				$obj = new Manufacturer($this->getOrCreateManufacturerId($polarisRef, (string) $this->GetMandatory($data, "name"), false));
				$origin = $this->getPSObjHash($obj);
				
				$this->updateProperty($obj, "name", $this->GetMandatory($data, "name"), NULL, "checkAttributeName");
				if ($this->Link->OPT_ALLOW_I_DESCRIPTIONS)
					$this->updateLgProperty($obj, "description", $data->description);				
				if ($this->Link->OPT_ALLOW_I_MDESCRIPTIONS)
					$this->updateLgProperty($obj, "meta_description", $data->meta);

				$obj->active = true;			
				if ($this->Link->OPT_ALLOW_I_PHOTOS)
					$this->ImportPhotoLogo("marque", _PS_MANU_IMG_DIR_, "manufacturers", "manufacturer", $obj, (string) $data->photos);
				
				if ($this->getPSObjHash($obj) != $origin)
					$obj->save();
			}
			else if (!empty($psId))
			{
				// Element non publié mais en cours sur le site, on active ou pas, selon
				$obj = new Manufacturer($psId);
				if ($obj->active)
				{
					$obj->active = false;
					$obj->save();
				}
			}		
		}
	}
	
