<?php
	/*
	* 2018 - VEGA Informatique
	*
	* NOTICE OF LICENSE
	*
	* This source file is subject to the Open Software License (OSL 3.0)
	* that is bundled with this package in the file LICENSE.txt.
	* It is also available through the world-wide-web at this URL:
	* http://opensource.org/licenses/osl-3.0.php
	*
	* DISCLAIMER
	*
	* Do not edit or add to this file if you wish to upgrade PolarisLink to newer
	* versions in the future. 
	*
	*  @author VEGA Informatique <contact@vega-info.fr>
	*  @copyright  2018 VEGA Informatique
	*  @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
	*/

	require_once(_PS_ROOT_DIR_ . "/modules/polarislink/classes/PolarisLinkImportBase.php");
	require_once(_PS_ROOT_DIR_ . "/modules/polarislink/classes/PolarisLinkImport_manufacturer.php");
	require_once(_PS_ROOT_DIR_ . "/modules/polarislink/classes/PolarisLinkImport_category.php");
	
	/*
	 * ex. : 
	  <product>
        <reference>albala</reference>
        <name>albala</name>
        <manufacturer ref="16384">arche</manufacturer>
        <category type="rfs" ref="835584">
            <category type="family" ref="606208">
                <category type="ray" ref="98304">
                    <name>maroquineries</name>
                </category>
                <name>sac</name>
            </category>
            <name>sac</name>
        </category>
        <category type="c1" ref="3833856">albia</category>
        <category type="season" ref="32768">permanent</category>
        <category type="collection" ref="32770">h14</category>
        <category type="collection" ref="327680">H12</category>
        <category type="collection" ref="65538">h15</category>
        <vat>20.00</vat>
        <ecotax>0.00</ecotax>
        <meta>chaussures femme arche emboitant confortable hiver 2016</meta>
        <description>Un emboitant stretch bi-matière asymétrique qui s'enfile.</description>
        <articles>
            <article sku="20015107">
                <size>36</size>
                <wp>65.25</wp>
                <sp>150.00</sp>
                <photos>42336257,42352641,42369025</photos>
            </article>
            <article sku="20015114">
                <size>37</size>
                <wp>65.25</wp>
                <sp>150.00</sp>
                <photos>42336257,42352641,42369025</photos>
            </article>
	  </product>
	 */
	class PolarisLinkImport_product extends PolarisLinkImportBase 
	{	
		private $HandledCategoriesRoot = null;
		private $Manufacturers = null;
		private $CategoryImporter = null;
		private $OrderedAttributesGroup;
		private $OrderedFeaturesGroup;
		private $SizeGridAttributesGroup;
		private $AttributesCache = null;
		private $FeaturesCache = null;

		private function addAttribute($id_attr, $xmlTag)
		{
			if ($id_attr > 0 && !empty($xmlTag))
				$this->OrderedAttributesGroup[$id_attr] = $xmlTag;
		}

		private function addFeature($id_feature, $xmlTag)
		{
			if ($id_feature > 0 && !empty($xmlTag))
				$this->OrderedFeaturesGroup[$id_feature] = $xmlTag;
		}
		
		protected function OnInit()
		{
			parent::OnInit();
			
			$this->HandledCategoriesRoot = array(
												$this->Link->CategoryForRFS,
												$this->Link->CategoryForCollection,
												$this->Link->CategoryForSeason,
												$this->Link->CategoryForCategory1,
												$this->Link->CategoryForCategory2,
												$this->Link->CategoryForCategory3,
												$this->Link->CategoryForCategory4,
												$this->Link->CategoryForTags
											);
			
			$this->Tags = $this->JobManager->getImporter("tag");
			$this->Manufacturers = $this->JobManager->getImporter("manufacturer");
			$this->CategoryImporter = $this->JobManager->getImporter("category");

			$this->SizeGridAttributesGroup = array();
			if ($this->Link->OPT_SIZE_GRID)
			{
				// Chargement des attributs...
				foreach (AttributeGroup::getAttributesGroups($this->Link->LangID) as $attrGr)
					$this->SizeGridAttributesGroup[mb_strtolower($attrGr["name"])] = $attrGr["id_attribute_group"];
			}

			$this->OrderedAttributesGroup = array();
			$this->addAttribute(intval($this->Link->AttrGroupForAttr0), "size");
			$this->addAttribute(intval($this->Link->AttrGroupForAttr1), "lvl1");
			$this->addAttribute(intval($this->Link->AttrGroupForAttr2), "lvl2");
			$this->addAttribute(intval($this->Link->AttrGroupForAttr3), "lvl3");
			$this->addAttribute(intval($this->Link->AttrGroupForAttr4), "lvl4");
			$this->addAttribute(intval($this->Link->AttrGroupForAttrG1), "g_lvl1");
			$this->addAttribute(intval($this->Link->AttrGroupForAttrG2), "g_lvl2");
			$this->addAttribute(intval($this->Link->AttrGroupForAttrG3), "g_lvl3");
			$this->addAttribute(intval($this->Link->AttrGroupForAttrG4), "g_lvl4");
			ksort($this->OrderedAttributesGroup);

			$this->OrderedFeaturesGroup = array();
			$this->addFeature(intval($this->Link->FeatureForAttr1), "lvl1");
			$this->addFeature(intval($this->Link->FeatureForAttr2), "lvl2");
			$this->addFeature(intval($this->Link->FeatureForAttr3), "lvl3");
			$this->addFeature(intval($this->Link->FeatureForAttr4), "lvl4");
			$this->addFeature(intval($this->Link->FeatureForAttrG1), "g_lvl1");
			$this->addFeature(intval($this->Link->FeatureForAttrG2), "g_lvl2");
			$this->addFeature(intval($this->Link->FeatureForAttrG3), "g_lvl3");
			$this->addFeature(intval($this->Link->FeatureForAttrG4), "g_lvl4");
			ksort($this->OrderedFeaturesGroup);
		}
		
		protected function BuildImage($product, $source)
		{
			$image = new Image();
			$image->id_product = (int)($product->id);
			$image->position = Image::getHighestPosition($product->id) + 1;
			// $image->legend[(int)$key] = $legend;			

			if (!Image::getCover($image->id_product)) {
				$image->cover = 1;
			} else {
				$image->cover = 0;
			}

			if (!$image->add()) 
			{
				$this->logError(sprintf("impossible d'ajouter une photo au produit %s", $product->id));
				return NULL;
			} 
			else 
			{
				$pfc = $image->getPathForCreation();
				if (!$new_path = $pfc) 
				{
					$this->logError(sprintf("impossible de créer le répertoire de destination de la photo pour le produit %s", $product->id));
					return NULL;
				}

				if (!@symlink(PolarisLinkAPI::getRelativePath($pfc, $source), $new_path.".jpg"))
				{
					$this->logError(sprintf("impossible d'ajouter une photo au produit %s : symlink impossible", $product->id));
					return NULL;
				}
			
				// Génération des miniatures !
				$imagesTypes = ImageType::getImagesTypes('products');
				$generate_hight_dpi_images = (bool)Configuration::get('PS_HIGHT_DPI');

				foreach ($imagesTypes as $imageType) 
				{
					if (!ImageManager::resize($source, $new_path.'-'.stripslashes($imageType['name']).'.'.$image->image_format, $imageType['width'], $imageType['height'], $image->image_format)) {
						$this->logError(sprintf("une erreur est survenue lors de la génération du thumbnail de l'image: %s", stripslashes($imageType['name'])));
						continue;
					}

					if ($generate_hight_dpi_images) {
						if (!ImageManager::resize($source, $new_path.'-'.stripslashes($imageType['name']).'2x.'.$image->image_format, (int)$imageType['width']*2, (int)$imageType['height']*2, $image->image_format)) {
							$this->logError(sprintf("une erreur est survenue lors de la génération du thumbnail haute définition de l'image: %s", stripslashes($imageType['name'])));
							continue;
						}
					}
				}
				
				// Plugin, watermark ?
				Hook::exec('actionWatermark', array('id_image' => $image->id, 'id_product' => $product->id));

				if (!$image->update()) 
				{
					$this->logError("erreur lors de la mise à jour du statut de l'image");
					return NULL;
				}

				// Associate image to shop from context
				$shops = Shop::getContextListShopID();
				$image->associateTo($shops);
				
				@unlink(_PS_TMP_IMG_DIR_.'product_'.(int)$product->id.'.jpg');
				foreach ($shops as $shop_id)
					@unlink(_PS_TMP_IMG_DIR_.'product_mini_'.(int)$product->id.'_'.$shop_id.'.jpg');
					
				return $image;
			}
		}
		
		public static function sortAttributesNatural($langId, $groupId)
		{
			$sizesSorter = array("tu" => 1, "xxxs" => 2, "xxs" => 3, "xs" => 4, "s" => 5, "m" => 6, "l" => 7, "xl" => 8, "xxl" => 9, "xxxl" => 10, "xxxxl" => 11);
			
			$attributes = array();
						
			foreach (AttributeGroup::getAttributes($langId, $groupId) as $attr)
			{
				$name = mb_strtolower(trim($attr["name"]));
				if (isset($sizesSorter[$name]))
					$name = $sizesSorter[$name];
				
				$attributes[$attr["id_attribute"]] = $name;
			}
						
			uasort($attributes, function($a, $b) {
									return strnatcmp($a, $b);
								});

			$position = 0;
			foreach ($attributes as $k => $v)
			{
				$attribute = new Attribute($k);
				$attribute->position = $position++;
				$attribute->save();
			}			
		}

		protected function getOrCreateAttribute($groupId, $names, $color = NULL, $position = NULL)
		{
			if ($this->AttributesCache == null)
			{
				// Chargement du cache d'attributs
				$id_lang = $this->Link->LangID;
				$attributesList = Db::getInstance()->executeS('
										SELECT DISTINCT ag.*, agl.*, a.`id_attribute`, a.color, a.position, al.`name`, agl.`name` AS `attribute_group`
										FROM `'._DB_PREFIX_.'attribute_group` ag
										LEFT JOIN `'._DB_PREFIX_.'attribute_group_lang` agl
											ON (ag.`id_attribute_group` = agl.`id_attribute_group` AND agl.`id_lang` = '.(int)$id_lang.')
										LEFT JOIN `'._DB_PREFIX_.'attribute` a
											ON a.`id_attribute_group` = ag.`id_attribute_group`
										LEFT JOIN `'._DB_PREFIX_.'attribute_lang` al
											ON (a.`id_attribute` = al.`id_attribute` AND al.`id_lang` = '.(int)$id_lang.')
										'.Shop::addSqlAssociation('attribute_group', 'ag').'
										'.Shop::addSqlAssociation('attribute', 'a').'
										ORDER BY agl.`name` ASC, a.`position` ASC
									', true, false);
				
				foreach ($attributesList as $attribute)
				{
					if (!isset($this->AttributesCache[$attribute["id_attribute_group"]]))
						$this->AttributesCache[$attribute["id_attribute_group"]] = array();
					
					$this->AttributesCache[$attribute["id_attribute_group"]][strtolower(trim($attribute["name"]))] = $attribute;
				}
			}
			
			if (isset($this->AttributesCache[$groupId][strtolower(trim($names[$this->Link->LangID]))]))
			{
				// Il existe 
				$attr = $this->AttributesCache[$groupId][strtolower(trim($names[$this->Link->LangID]))];
				
				// On vérifie la couleur
				if (!empty($color) && $color != $attr["color"])
				{
					$attribute = new Attribute($attr["id_attribute"]);
					$attribute->color = $color;
					$attribute->save();
					// Mise à jour du cache
					$this->AttributesCache[$groupId][strtolower(trim($names[$this->Link->LangID]))]["color"] = $color;
				}
				
				// On vérifie la position
				/*
				 * Nous ne pouvons pas procéder ainsi car cela pose trop de problème !
				 *
				 *
				if (!empty($position) && $position != $attr["position"])
				{
					$attribute = new Attribute($attr["id_attribute"]);
					$attribute->position = $position;
					$attribute->save();
					// Mise à jour du cache
					$this->AttributesCache[$groupId][strtolower(trim($names[$this->Link->LangID]))]["position"] = $position;
				}
				*/
				
				return $attr["id_attribute"];
			}
			
			// L'attribut n'existe pas ? On le crée !
			$attribute = new Attribute();
			$attribute->id_attribute_group = $groupId;
			$attribute->position = $position;
			$attribute->color = $color;
			$attribute->name = $names;
			$attribute->save();
			
			// Mise à jour du cache
			$this->AttributesCache[$groupId][strtolower(trim($names[$this->Link->LangID]))] = array(
																								"id_attribute" => $attribute->id,
																								"id_attribute_group" => $groupId,
																								"position" => $position,
																								"color" => $color
																							);
			return $attribute->id;
		}

		protected function getOrCreateFeature($featureId, $names)
		{
			if ($this->FeaturesCache == null)
			{
				// Chargement du cache d'attributs
				$id_lang = $this->Link->LangID;
				$featuresValueList = Db::getInstance()->executeS('
										SELECT DISTINCT 
												fv.id_feature, fvl.id_feature_value, fvl.value
										FROM `'._DB_PREFIX_.'feature_value_lang` fvl
										LEFT JOIN `'._DB_PREFIX_.'feature_value` fv
											ON (fv.`id_feature_value` = fvl.`id_feature_value` AND fvl.`id_lang` = '.(int)$id_lang.')
									', true, false);
				
				foreach ($featuresValueList as $featureValue)
				{
					if (!isset($this->FeaturesCache[$featureValue["id_feature"]]))
						$this->FeaturesCache[$featureValue["id_feature"]] = array();
					
					$this->FeaturesCache[$featureValue["id_feature"]][strtolower(trim($featureValue["value"]))] = $featureValue;
				}
			}
			
			if (isset($this->FeaturesCache[$featureId][strtolower(trim($names[$this->Link->LangID]))]))
			{
				// Il existe 
				$attr = $this->FeaturesCache[$featureId][strtolower(trim($names[$this->Link->LangID]))];				
				return $attr["id_feature_value"];
			}
						
			// L'attribut n'existe pas ? On le crée !
			$featureValue = new FeatureValue();
			$featureValue->id_feature = $featureId;
			$featureValue->value = $names;
			$featureValue->save();
			
			// Mise à jour du cache
			$this->FeaturesCache[$featureId][strtolower(trim($names[$this->Link->LangID]))] = array(
																								"id_feature_value" => $featureValue->id,
																								"id_feature" => $featureId
																							);
			return $featureValue->id;
		}
		
		
		protected function associateProductToPolarisRef($polarisRef, $productId, $productAttributeId = 0)
		{
			$productSupplier = ProductSupplier::getIdByProductAndSupplier($productId, $productAttributeId, $this->Link->SupplierID);
			if ($productSupplier)
			{
				$productSupplier = new ProductSupplier($productSupplier);
				if ($productSupplier->product_supplier_reference != $polarisRef || $productSupplier->id_currency != $this->Link->CurrencyID)
				{
					$productSupplier->product_supplier_reference = $polarisRef;
					$productSupplier->id_currency = $this->Link->CurrencyID;
					$productSupplier->save();
				}
			}
			else
			{
				$productSupplier = new ProductSupplier();
				$productSupplier->id_supplier = $this->Link->SupplierID;
				$productSupplier->id_product = $productId;
				$productSupplier->id_product_attribute = $productAttributeId;
				$productSupplier->product_supplier_reference = $polarisRef;
				$productSupplier->id_currency = $this->Link->CurrencyID;
				$productSupplier->add();
			}
		}
		
		/*
			Détermine si la catégorie $id est la descendante (directe ou non) de $category_id
		*/
		public static function isSubCategoryOf(&$categories, $category_id, $id, $count = 0)
		{
			if ($count > 50)	// On évite les références cycliques !
				return false;
			
			if (empty($category_id))
				return false;
			
			if ($category_id == $id)
				return true;
			
			if (!isset($categories[$id]))
				return false;
			
			$parent_id = $categories[$id]["id_parent"];			
			if (empty($parent_id))
				return false;
			
			return self::isSubCategoryOf($categories, $category_id, $parent_id, $count + 1);
		}
		
		/*
			Détermine toutes les catégories à ajouter au produit pour la descendance
			de la catégorie spécifiée
		*/
		protected function categoryAddAllParents($category_id, &$categories, $max = 20)
		{
			if ($max <= 0)
				return;	// on empêche les cycles infinis

			$categories[$category_id] = $category_id;
			
			$psCategories = $this->CategoryImporter->GetCategoriesById();
			$parentId = isset($psCategories[$category_id]) ? $psCategories[$category_id]["id_parent"] : 0;
			if ($parentId != 0 && $parentId != (int)Configuration::get('PS_ROOT_CATEGORY') && !in_array($parentId, $this->HandledCategoriesRoot))
				self::categoryAddAllParents($parentId, $categories, $max-1);
		}
		
		public function reassociateAttributes($combination, $ids_attribute)
		{
			$result = Db::getInstance()->delete('product_attribute_combination', '`id_product_attribute` = '.(int)$combination->id);

			if ($result && !empty($ids_attribute)) {
				$sql_values = array();
				foreach ($ids_attribute as $value) {
					$sql_values[] = '('.(int)$value.', '.(int)$combination->id.')';
				}

				$result = Db::getInstance()->execute('
					INSERT INTO `'._DB_PREFIX_.'product_attribute_combination` (`id_attribute`, `id_product_attribute`)
					VALUES '.implode(',', $sql_values)
				);
			}
			return $result;
		}

		protected function OnImportData($data)
		{
			if (!$this->Link->OPT_ALLOW_I_CATALOG)
				return;	// Rien à faire ici dans ce cas là !
			
			if ($data->articles == null || count($data->articles) == 0)
				return;	// Rien à importer !
			
			// On se positionne sur le bon shop !
			$this->SetCatalogContext();
			
			// Localisation des produits concernés ...
			$references = array();
			$eans = array();
			$refPublished = 0;
			$minPA = 0;
			$minPV = 0;
			$photosList = array();
			
			foreach ($data->articles->article as $article)
			{
				$sku = (string) $article["sku"];
				if (empty($sku))
					throw new Exception("article sans SKU détecté");
				
				$references[] = $sku;
				if ((int) $article["deleted"] == 0)
				{
					$refPublished += 1;
					// On regarde les prix ?
					$pa = floatval((string) $article->wp);
					if ($minPA == 0 || ($pa > 0 && $minPA > $pa))
						$minPA = $pa;
					
					$pv = floatval((string) $article->sp);
					if ($minPV == 0 || ($pv > 0 && $minPV > $pv))
						$minPV = $pv;
					
					$ean = (string) $article["ean"];
					if (!empty($ean))
						$eans[] = $ean;	

					if (!empty((string) $article->photos))
						$photosList = array_merge($photosList, $this->GetReferencesFromList((string) $article->photos));
				}
			}
			$references = join(',', $references);
			$eans = join(',', $eans);
			
			// On ne localise QUE par EAN (et pas par GENCODE) pour le catalogue, sinon catastrophe !!
			// (on ne met à jour que les références que le pont a créé, le reste c'est pour reconnaitre pour les stocks et les prix)
			$isOnCreation = false;
			$product = $this->GetProduct($this->GetProductCombinationFor($references, NULL, false, false, true));
			
			if ($refPublished == 0)
			{				
				// Suppression complète du modèle ...
				if ($product != NULL) // produit trouvé 
				{
					if ($this->Link->OPT_DELETE_WHEN_UNPUBLISHED)
					{
						if (!$product->delete())
						{
							// Problème à la suppression
							if ($product->active != false)
							{
								// On désactive le modèle, si ce n'est déjà fait
								$product->active = false;
								$product->save();
							}
							
							// Puis on essaye de supprimer le stock physique si jamais c'est ça qui pose soucis ...
							if (StockAvailable::dependsOnStock($product->id))
							{
								$stockManager = StockManagerFactory::getManager();
								$stocks = new PrestaShopCollection('Stock');
								$stocks->where('id_product', '=', $product->id);
								
								foreach ($stocks as $stock)
								{
									$qty = $stock->physical_quantity;
									$warehouse = new Warehouse($stock->id_warehouse);
									
									if ($qty > 0)
										$stockManager->removeProduct($stock->id_product, $stock->id_product_attribute, $warehouse, $qty, Configuration::get('PS_STOCK_MVT_INC_REASON_DEFAULT'), 0);
									else if ($qty < 0)
										$stockManager->addProduct($stock->id_product, $stock->id_product_attribute, $warehouse, $qty * -1, Configuration::get('PS_STOCK_MVT_INC_REASON_DEFAULT'), 0);
								}								
							}
							
							// Enfin, on recommence, en cas d'échec, on le signale
							if (!$product->delete())
								throw new Exception(sprintf("erreur à la suppression du produit n°%d", $product->id));
						}
					}
					else
					{
						if ($product->active != false)
						{
							$product->active = false;
							$product->save();
						}
					}
				}				
			}
			else
			{
				if ($product == NULL)
				{
					$product = new Product();
					$product->active = false;
					$isOnCreation = true;
					
					$product->reference = checkProductRef((string) $this->GetMandatory($data, "reference"));
					$product->name = checkProductName($this->getLgProperty($this->GetMandatory($data, "name"), true));										
										
					$product->link_rewrite = array();
					foreach ($product->name as $k => $v)
						$product->link_rewrite[$k] = Tools::link_rewrite($v);
				}
				
				$productHash = $this->buildHashValues($product);
				
				$polarisRef = $this->GetFirstReferencesFromList($references);
				$product->id_supplier = $this->Link->SupplierID;
				$product->supplier_reference = $polarisRef;
				
				$this->updateProperty($product, "reference", $this->GetMandatory($data, "reference"), null, 'checkProductRef');
				
				if (!empty($eans))
					$product->ean13 = checkEAN13($this->GetFirstReferencesFromList($eans));
				
				if ($this->Link->OPT_ALLOW_I_NAME)
					$this->updateLgProperty($product, "name", $this->GetMandatory($data, "name"), null, false, 'checkProductName');
				
				// Prix et taxes
				if (!empty((string) $data->vat))
				{
					// On essaye par règle ...
					$id_tax_rule = (string) $data->vat["rule"];
					if (!$this->JobManager->isTaxesRulesExists($id_tax_rule))
					{
						$this->logError(sprintf("la règle de taxes n°%s n'existe pas !", $id_tax_rule));
						return;	// pas d'exception car on ne veut pas y mettre tout le détail du xml
					}
										
					if (empty($id_tax_rule))
					{					
						// Pas trouvé ? On essaye de deviner la règle ...
						$id_country = NULL;
						if (!empty((string) $data->vat["country"]))
							$id_country = (int) $this->JobManager->getCountryForISO((string) $data->vat["country"]);
										
						// Pas de pays, on prend celui du magasin par défaut !
						if (empty($id_country))
							$id_country = (int) Configuration::get('PS_COUNTRY_DEFAULT');
						
						$id_tax_rule = $this->JobManager->getTaxesGroupFor($id_country, (string) $data->vat);
					}
					
					if (empty($id_tax_rule))
						throw new Exception(sprintf("impossible de localiser la taxe de %s pour paramétrer le produit %s", (string) $data->vat, $product->reference));
					
					$product->id_tax_rules_group = $id_tax_rule;
				}
								
				$this->updateProperty($product, "ecotax", $this->GetOptional($data, "ecotax"));
				$product->price = checkPrice($minPV);
				$product->wholesale_price = checkPrice($minPA);
				
				$this->updateProperty($product, "weight", $this->GetOptional($data, "weight"));
				$this->updateProperty($product, "width", $this->GetOptional($data, "width"));
				$this->updateProperty($product, "height", $this->GetOptional($data, "height"));
				$this->updateProperty($product, "depth", $this->GetOptional($data, "depth"));
				
				if ($this->Link->OPT_ALLOW_I_MDESCRIPTIONS)
					$this->updateLgProperty($product, "meta_description", $this->GetOptional($data, "meta"), null, false, 'checkMetaDescription');
				if ($this->Link->OPT_ALLOW_I_DESCRIPTIONS)
					$this->updateLgProperty($product, "description", $this->GetOptional($data, "description"), null, false, 'checkDescription');
				
				// Liaison marque
				if (!empty((string) $data->manufacturer["ref"]))
					$product->id_manufacturer = $this->Manufacturers->getOrCreateManufacturerId((string) $data->manufacturer["ref"], (string) $data->manufacturer);
				
				if ($this->buildHashValues($product) != $productHash)
					$product->save();
				
				// On vérifie la référence fournisseur
				$this->associateProductToPolarisRef($polarisRef, $product->id);
				
				$no_group = intval($this->GetOptional($data, "", "group"));
				if ($no_group > 0)
				{
					// On vérifie que le modèle appartient au groupe
					$db = Db::getInstance();
					$no_group_actuel = NULL;					
					
					foreach ($db->executeS("SELECT id_group FROM "._DB_PREFIX_."polarislink_products_group WHERE id_product = ".$product->id) as $row)
						$no_group_actuel = intval($row["id_group"]);
					
					if ($no_group_actuel != $no_group)
					{
						if ($no_group_actuel == 0)
							$db->query("INSERT INTO "._DB_PREFIX_."polarislink_products_group (id_product, id_group) VALUES (".$product->id.", ".$no_group.")");
						else
							$db->query("UPDATE "._DB_PREFIX_."polarislink_products_group SET no_group = ".$no_group." WHERE id_product = ".$product->id);
					}
				}
				
				// Association avec	les catégories ...
				$currentHandledCategories = array();				
				
				$psCategories = $this->CategoryImporter->GetCategoriesById();
				foreach ($product->getCategories() as $currentCategory)
				{
					foreach ($this->HandledCategoriesRoot as $rootCategoryHandledByPolaris)
					{
						if (self::isSubCategoryOf($psCategories, $rootCategoryHandledByPolaris, $currentCategory))
							$currentHandledCategories[$currentCategory] = $currentCategory;
					}
				}
				
				// Construction de l'association
				$defaultCategory = null; // (rfs)
				$handledCategoriesToHave = array();
				foreach ($data->category as $category)
				{
					$psId = $this->CategoryImporter->ImportData($category);
					if (!empty($psId))
					{
						// On remonte toutes les catégories jusqu'au sommet
						self::categoryAddAllParents($psId, $handledCategoriesToHave);
						
						// On se note la RFS qui servira de catégorie par défaut
						if (isset($category["type"]) && $category["type"] == "rfs")
							$defaultCategory = $psId;
					}
				}
				
				if ($this->Link->MethodTags == 1)
				{
					// Dans ce mode, les tags sont des catégories !!
					foreach ($data->tag as $tag)
					{
						$psId = $this->CategoryImporter->ImportData($tag, "grouptag", "tag", "grtag");
						$handledCategoriesToHave[$psId] = $psId;
					}
				}
				
				ksort($currentHandledCategories);
				ksort($handledCategoriesToHave);				
				
				if (serialize($currentHandledCategories) != serialize($handledCategoriesToHave))
				{					
					foreach ($currentHandledCategories as $category_id)
					{
						if (!isset($handledCategoriesToHave[$category_id]))
							$product->deleteCategory($category_id);
					}
					
					$add = array();
					foreach ($handledCategoriesToHave as $category_id)
					{
						if (!isset($currentHandledCategories[$category_id]))
							$add[] = $category_id;
					}
					
					if (count($add) > 0)
						$product->addToCategories($add);
				}
				
				// Vérification de la catégorie par défaut
				if (!empty($product->id_category_default))
				{
					$category = new Category($product->id_category_default);
					if (empty($category->id))
						$product->id_category_default = NULL;
				}
				
				// Sélection de la catégorie par défaut ...
				if (!$this->Link->AutoCategory)
				{
					// Pas de catégorie par défaut = A classer !
					if (empty($product->id_category_default) && !empty($this->Link->SortingCategoryID))
					{
						$product->addToCategories($this->Link->SortingCategoryID);
						
						$defaultCategory = $this->Link->SortingCategoryID;
						if ($product->id_category_default != $defaultCategory)
						{
							$product->id_category_default = $defaultCategory;
							$product->save();
						}
					}
				}
				else
				{
					// Catégorie automatique !
					if ( ($this->Link->DefaultCategoryRoot != 0 &&
						  $this->Link->DefaultCategoryRoot != Configuration::get('PS_ROOT_CATEGORY')) || empty($product->id_category_default))
					{
						// Sélection de la catégorie par défaut 
						// Si ca correspond à la RFS, on l'a déjà calculé !!
						if ($this->Link->DefaultCategoryRoot != $this->Link->CategoryForRFS)
						{
							$psCategories = $this->CategoryImporter->GetCategoriesById();
							foreach ($product->getCategories() as $currentCategory)
							{
								if (self::isSubCategoryOf($psCategories, $this->Link->DefaultCategoryRoot, $currentCategory))
									$defaultCategory = $currentCategory;
							}
						}
						
						if ($product->id_category_default != $defaultCategory)
						{
							$product->id_category_default = $defaultCategory;
							$product->save();
						}
					}
				}
				
				// Tags ?
				if (!empty($data->tag))
					$this->Tags->addToProduct($product, $data->tag);
				
				// On s'occupe des photos				
				if ($this->Link->OPT_ALLOW_I_PHOTOS)
				{					
					$currentPhotos = array();
					$currentPhotosByPoId = array();
					
					foreach ($product->getImages($this->Link->LangID) as $img)
					{
						$currentPhoto = array(
												"psId" => $img["id_image"],
												"poId" => NULL,
												"order" => 0,
												"c_order" => count($currentPhotos) + 1,
												"used" => false
											);
																		
						$imgPath = _PS_PROD_IMG_DIR_.Image::getImgFolderStatic($img["id_image"]).$img["id_image"].".jpg";
						if (is_link($imgPath))
						{
							$currentPhoto["poId"] = basename(readlink($imgPath), ".jpg");
							$currentPhotosByPoId[$currentPhoto["poId"]] = $currentPhoto["psId"];
						}						
						
						$currentPhotos[$currentPhoto["psId"]] = $currentPhoto;
					}
					
					// Création des photos produits manquantes									
					$order = 1;
					foreach ($photosList as $poId)
					{
						$poId = trim($poId);
						if (!isset($currentPhotosByPoId[$poId]))
						{							
							$polarisPhoto = PolarisLinkAPI::GetPolarisPhotoPathFor($this->Link->LinkID, $poId);
							if (file_exists($polarisPhoto))
							{								
								// La photo Polaris existe, il ne reste qu'à faire l'association !!
								// Pour ça, on ajoute la photo au modèle
								$img = $this->BuildImage($product, $polarisPhoto);
								if (!empty($img))
								{
									$currentPhotos[$img->id] = array(
																		"psId" => $img->id,
																		"poId" => $poId,
																		"order" => $order++,
																		"c_order" => 0,
																		"used" => true
																	);																
									$currentPhotosByPoId[$poId] = $img->id;
								}
							}
						}
						else
						{
							$currentPhotos[$currentPhotosByPoId[$poId]]["used"] = true;
							if ($currentPhotos[$currentPhotosByPoId[$poId]]["order"] == 0)
								$currentPhotos[$currentPhotosByPoId[$poId]]["order"] = $order++;
						}
					}
					
					// Changement des positions des photos
					foreach ($currentPhotos as $photo)
					{
						if ($photo["order"] != $photo["c_order"])
						{
							$img = new Image($photo["psId"]);
							$img->position = $photo["order"];
							$img->save();
						}
					}
				}
				
				// Chargement des déclinaisons actuelles				
				if ($this->Link->OPT_SIZE_GRID)
				{					
					$id_group_attribute_sizegrid = $this->Link->AttrGroupForAttr0;
					$grille_taille = (string) $this->GetOptional($data, "sizegrid");
					if (!empty($grille_taille))
					{
						$grille_taille = mb_strtolower($grille_taille);						
						if (isset($this->SizeGridAttributesGroup[$grille_taille]))
							$id_group_attribute_sizegrid = $this->SizeGridAttributesGroup[$grille_taille];
					}
				}

				$currentCombinations = array();
				$currentCombinationsBySupplierRef = array();
				$lvlKeyToCombinationId = array();				
				
				$sql = "SELECT id_product_attribute, product_supplier_reference
						FROM "._DB_PREFIX_."product_supplier 
					WHERE id_product_attribute != 0 AND id_product=".(int)$product->id." AND id_supplier IN (".join(',', array($this->Link->SupplierID, $this->Link->SupplierBackupID)).")";
				$assocs = Db::getInstance()->executeS($sql, true, false);
				/*if ($assocs === false)
					throw new Exception("Erreur au retrait des références fournisseurs");*/
				foreach ($assocs as $row)
					$currentCombinationsBySupplierRef[$row["product_supplier_reference"]] = (int) $row["id_product_attribute"];

				$sql = "SELECT pa.id_product_attribute AS id, pa.supplier_reference, 
								GROUP_CONCAT(CONCAT_WS(':', a.id_attribute_group, a.id_attribute) ORDER BY a.id_attribute_group ASC SEPARATOR ',') AS keylvl
						FROM "._DB_PREFIX_."product_attribute pa
							JOIN "._DB_PREFIX_."product_attribute_combination pac ON (pac.id_product_attribute = pa.id_product_attribute)
							JOIN "._DB_PREFIX_."attribute a ON (pac.id_attribute = a.id_attribute)
						WHERE pa.id_product=".(int)$product->id." GROUP BY pa.id_product_attribute";				
				$assocs = Db::getInstance()->executeS($sql, true, false);
				/*if (!$assocs === false)
					throw new Exception("Erreur au retrait des déclinaisons existantes");*/

				foreach ($assocs as $row)
				{
					$currentCombinations[(int) $row["id"]] = false;
					$lvlKeyToCombinationId[$row["keylvl"]] = (int) $row["id"];

					if (!isset($currentCombinationsBySupplierRef[$row["supplier_reference"]]))
						$currentCombinationsBySupplierRef[$row["supplier_reference"]] = (int) $row["id_product_attribute"];
				}

				// Caractéristique ?
				if (count($this->OrderedFeaturesGroup) > 0)
				{
					$currentFeatures = [];
					foreach ($product->getFeatures() as $f)
						$currentFeatures[$f["id_feature"]] = $f["id_feature_value"];

					foreach ($this->OrderedFeaturesGroup as $k => $v)
					{
						foreach ($data->articles->article as $article)
						{
							$value = $article->$v;
							if (!empty($value))
							{
								$feature_id = $this->getOrCreateFeature($k, checkAttributeName($this->getLgProperty($value, true)));

								if (!isset($currentFeatures[$k]) || $feature_id != $currentFeatures[$k])
								{
									Db::getInstance()->execute('DELETE FROM `'._DB_PREFIX_.'feature_product` WHERE `id_product` = '.(int)$product->id.' AND `id_feature` = '.(int)$k);
           							$product->addFeaturesToDB($k, $feature_id);		
								}

								// On s'arrête au premier !
								break;
							}
						}
					}
				}				
				
				// On construit les déclinaisons ici				
				foreach ($data->articles->article as $article)
				{
					if (empty((string) $article["deleted"]))
					{
						$cPolarisRef = $this->GetFirstReferencesFromList($this->GetMandatory($article, "", "sku"));
						
						// Construction des liens attributs
						$lvlKey = "";
						$attrs = array();
						foreach ($this->OrderedAttributesGroup as $k => $v)
						{
							$value = $article->$v;
							if (!empty($value))
							{
								$id_group_attribute = $k;
								if ($this->Link->OPT_SIZE_GRID && $v == "size")
									$id_group_attribute = $id_group_attribute_sizegrid;
								
								$attr = $this->getOrCreateAttribute($id_group_attribute, checkAttributeName($this->getLgProperty($value, true)), (string) $value["color"], (string) $value["position"]);
								$lvlKey .= ",".$id_group_attribute.":".$attr;
								$attrs[] = $attr;
							}						
						}
						
						if (strlen($lvlKey) > 1)
						{
							$lvlKey = substr($lvlKey, 1);
							$isNew = false;
							$reAssociateAttribute = false;
							
							// Si la déclinaison existe dans l'état, on la charge et la marque comme étant utilisée
							$newCombination = null;

							// Recherche par attributs ...
							if (isset($lvlKeyToCombinationId[$lvlKey]))
							{
								$id = $lvlKeyToCombinationId[$lvlKey];
								$newCombination = new Combination($id);
								$currentCombinations[$id] = true;
							}
							else 
							{
								// Pas trouvé ? Recherche par équivalence de référence fournisseur ...
								foreach ($this->GetReferencesFromList($this->GetMandatory($article, "", "sku")) as $supplierRef)
								{
									if (isset($currentCombinationsBySupplierRef[$supplierRef]))	
									{
										$id = $currentCombinationsBySupplierRef[$supplierRef];										
										if (!$currentCombinations[$id])	// que si la combinaison n'a pas été utilisée pour une autre association
										{											
											$newCombination = new Combination($id);
											$currentCombinations[$id] = true;
											$reAssociateAttribute = true;
											// trouvé ?
											break;
										}
									}
								}								
							}

							if (!$newCombination)
							{								
								// Pas trouvé ?								
								// Création de la déclinaison								
								$newCombination = new Combination();
								$newCombination->id_product = $product->id;	
								$isNew = true;
							}
							
							$cmbHash = $this->buildHashValues($newCombination);

							$newCombination->reference = $cPolarisRef;
							$newCombination->supplier_reference = $cPolarisRef;
							
							$c_ean13 = (string) $this->GetOptional($article, "", "ean");
							if (!empty($c_ean13))
								$newCombination->ean13 = checkEAN13($this->GetFirstReferencesFromList($c_ean13));
							
							$newCombination->price = checkPrice((!empty((string) $article->sp)) ? (float) $article->sp - $minPV : 0);
							$newCombination->wholesale_price = checkPrice((!empty((string) $article->wp)) ? (float) $article->wp - $minPA : 0);
							
							/*if ($newCombination->wholesale_price != 0)
								throw new Exception("Wholesale: ".$newCombination->wholesale_price);*/

							if ($this->buildHashValues($newCombination) != $cmbHash)
								$newCombination->save();

							if ($isNew)
								$newCombination->setAttributes($attrs);
							else if ($reAssociateAttribute)
								$this->reassociateAttributes($newCombination, $attrs);
							
							$this->associateProductToPolarisRef($cPolarisRef, $product->id, $newCombination->id);
							
							// Photos !
							if (!empty((string) $article->photos) && $this->Link->OPT_ALLOW_I_PHOTOS)
							{
								$havePhotos = array();
								foreach ($newCombination->getWsImages() as $img)
									$havePhotos[] = $img["id"];								
								sort($havePhotos, SORT_NUMERIC);

								$wantedPhoto = array();								
								foreach ($this->GetReferencesFromList((string) $article->photos) as $poId)
								{
									if (isset($currentPhotosByPoId[$poId]))
									{
										$psId = $currentPhotosByPoId[$poId];
										$wantedPhoto[] = $psId;										
									}			
								}
								
								sort($wantedPhoto, SORT_NUMERIC);
								
								if (join(',', $havePhotos) != join(',', $wantedPhoto))
									$newCombination->setImages($wantedPhoto);								
							}
						}
					}										
				}
				
				// Enfin, on efface les déclinaisons qui n'existe plus !!
				foreach ($currentCombinations as $id => $exists)
				{
					if (!$exists)
						$product->deleteAttributeCombination($id);
				}
				
				if ($this->Link->OPT_ALLOW_I_PHOTOS)
				{
					// Suppression des photos en trop ...
					$firstPhoto = null;
					foreach ($currentPhotos as $photo)
					{
						if (!$photo["used"])
						{
							$img = new Image($photo["psId"]);
							if ($img != NULL)
								$img->delete();
						}
						else if (empty($firstPhoto))
							$firstPhoto = $photo["psId"];
							
					}
					
					if (empty($product->getCoverWs()))
						$product->setCoverWs($firstPhoto);
				}
				
				// On active le produit !
				if ($isOnCreation && $product->active == false && $this->Link->OPT_ACTIVE_PUBLISHED)
				{
					$product->active = true;
					$product->save();
				}
				
				// Déclinaison par défaut ?					
				$autoDetectDefault = (int)Product::getDefaultAttribute($product->id, 0, true);
				if (!empty($product->cache_default_attribute))
				{
					// Vérification ...
					$combination = new Combination($product->cache_default_attribute);
					if (!$combination->default_on)
						$product->cache_default_attribute = NULL;
				}
				
				if (empty($product->cache_default_attribute) || empty($autoDetectDefault))
				{
					$product->deleteDefaultAttributes();
					
					foreach (Product::getProductAttributesIds($product->id) as $attr)
					{
						$combination = new Combination($attr["id_product_attribute"]);
						$combination->default_on = 1;
						$combination->save();
						
						$product->setDefaultAttribute($combination->id);
						break;
					}
				}				
			}
		}		
	}
	