<?php
	/*
	* 2018 - VEGA Informatique
	*
	* NOTICE OF LICENSE
	*
	* This source file is subject to the Open Software License (OSL 3.0)
	* that is bundled with this package in the file LICENSE.txt.
	* It is also available through the world-wide-web at this URL:
	* http://opensource.org/licenses/osl-3.0.php
	*
	* DISCLAIMER
	*
	* Do not edit or add to this file if you wish to upgrade PolarisLink to newer
	* versions in the future. 
	*
	*  @author VEGA Informatique <contact@vega-info.fr>
	*  @copyright  2018 VEGA Informatique
	*  @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
	*/

	require_once(_PS_ROOT_DIR_ . "/modules/polarislink/classes/PolarisLinkImportBase.php");	
	require_once("PolarisLinkImport_warehouse.php");	
		
	/*
	 * ex. : <stock sku="11" ean="22" warehouse="1" [w_price="5.00"] [s_price="10.00"]>10</stock>
	 */
	class PolarisLinkImport_stock extends PolarisLinkImportBase 
	{
		private $stockManager = null;		
		private $stock_reason = null;
		private $warehouses = null;
		
		protected function OnInit()
		{
			$this->stockManager = StockManagerFactory::getManager();			
			$this->stock_reason = Configuration::get('PS_STOCK_MVT_INC_REASON_DEFAULT');
			$this->warehouses = $this->JobManager->getImporter("warehouse");
		}
		
		protected function OnImportData($data)
		{
			if (!$this->Link->OPT_ALLOW_I_STOCKS && !$this->Link->OPT_ALLOW_I_SALE_PRICES)
				return;	// Rien à faire ici dans ce cas là !
			
			// Ok, nous cherchons donc le produit prestashop correspondant
			// à la référence donnée
			$product = $this->GetProductCombinationFor((string) $data["sku"], 
								$this->Link->OPT_USE_EAN_STOCK ? (string) $data["ean"] : null, false, true);
								
			if ($product == NULL) // produit non trouvé 
				return;
				
			// Chargement du produit
			$c_product = $this->GetProduct($product["id_product"]);
			$cmb_product = null;
			if (!empty($product["id_product_attribute"]))
				$cmb_product = new Combination($product["id_product_attribute"]);
			
			// Prix d'achat et prix de vente ?
			if ($this->Link->OPT_ALLOW_I_SALE_PRICES)
			{
				$this->SetPricesContext();
				$pa = -1;
				$pv = -1;
				if (isset($data["w_price"]))
					$pa = (float) $data["w_price"];
				
				if (isset($data["s_price"]))
					$pv = (float) $data["s_price"];
				
				// Changement du prix fournisseur Polaris
				if ($pa >= 0)
				{
					$suppliers = new PrestaShopCollection('ProductSupplier');
					$suppliers->sqlWhere("id_product = ".$product["id_product"].
									" AND id_product_attribute = ".$product["id_product_attribute"].
									" AND id_supplier = ".$this->Link->SupplierID);
					foreach ($suppliers as $supplier)
					{
						if ($supplier->product_supplier_price_te != $pa)
						{
							$supplier->product_supplier_price_te = $pa;
							$supplier->save();
						}
					}
				}			
				
				// Changement des prix dans les modèles
				if ($pa >= 0 || $pv > 0)
				{
					// Les prix sont spécifiés, il faut les contrôler ...				
					if (!empty($cmb_product))
					{
						$pv -= $c_product->price;
						if ($cmb_product->wholesale_price != $pa || ($this->Link->OPT_ALLOW_I_SALE_PRICES && $cmb_product->price != $pv))
						{
							if ($pa >= 0)
								$cmb_product->wholesale_price = $pa;
							
							if ($this->Link->OPT_ALLOW_I_SALE_PRICES)
								$cmb_product->price = $pv;
							$cmb_product->save();
						}
					}
					
					if ($pa >= 0 && $c_product->wholesale_price != $pa)
					{
						$c_product->wholesale_price = $pa;
						$c_product->save();
					}
									
					if ($this->Link->OPT_ALLOW_I_SALE_PRICES && $pv > 0 && $cmb_product == null && $c_product->price != $pv)
					{
						$c_product->price = $pv;
						$c_product->save();
					}
				}
			}
			
			if ($this->Link->OPT_ALLOW_I_STOCKS)
			{
				$this->SetCatalogContext();
				$use_adv_stock = Configuration::get('PS_ADVANCED_STOCK_MANAGEMENT') && $this->Link->OPT_USE_ADV_STOCK;						
				$depends_on_stock = StockAvailable::dependsOnStock($product["id_product"]);	
				
				if ($use_adv_stock)
				{
					// On utilise l'option des stocks avancés, on s'assure que le produit est bien dans 
					// ce mode...				
					if (!$depends_on_stock)
					{
						// On vide le stock manuel et entrepôt déjà fait (on recommence quoi) ...
						Db::getInstance()->update('stock_available', array('quantity' => 0), 'id_product = '.$product["id_product"]);
						Db::getInstance()->update('stock', array('physical_quantity' => 0, 'usable_quantity' => 0), 'id_product = '.$product["id_product"]);
											
						$c_product->setAdvancedStockManagement(1);					
						StockAvailable::setProductDependsOnStock($product["id_product"]);
					}
				}
				else
				{
					// On n'utilise plus l'option des stocks avancés, on s'assure que le produit n'est plus dans
					// ce mode ...
					if ($depends_on_stock)
					{										
						$c_product->setAdvancedStockManagement(0);					
						StockAvailable::setProductDependsOnStock($product["id_product"], 0);
					}
				}
				
				// On cumule ici tous les entrepôts
				$warehouses = array();
				$total_stock = 0;
				foreach ($data->warehouse as $warehouse_data)
				{				
					$qty = (int) $warehouse_data;
					$total_stock += $qty;
					
					if ($use_adv_stock)
					{
						$code = (string) $warehouse_data["code"];
						if ($code !== "0" && empty($code))
							throw new Exception("l'attribute [code] n'a pas été spécifié alors qu'attendu");

						$warehouse = $this->warehouses->getOrCreateWarehouse($code);					
						if (!isset($warehouses[$warehouse->id]))
							$warehouses[$warehouse->id] = array("wh" => $warehouse, "stock" => 0);
					
						$warehouses[$warehouse->id]["stock"] += $qty;
					}
				}
								
				if ($use_adv_stock)
				{
					// Stocks avancés
					$done = false;
					foreach ($warehouses as $wh)
					{
						// Cherchons l'ancienne valeur communiquée			
						$warehouse = $wh['wh'];
						$current_stock = $this->stockManager->getProductPhysicalQuantities($product["id_product"], $product["id_product_attribute"], array($warehouse->id));
						
						// On rajoute les commandes non encore importées sous Polaris
						// TODO : On rajoute les commandes non encore importées sous Polaris			
						
						// Stock à insérer 
						$stock = $wh['stock'];
						$qty = $stock - $current_stock;
						
						if ($qty != 0)
						{
							// On s'assure que le produit est bien associé à l'entrepôt
							if (empty(WarehouseProductLocation::getIdByProductAndWarehouse($product["id_product"], $product["id_product_attribute"], $warehouse->id)))
								Warehouse::setProductLocation($product["id_product"], $product["id_product_attribute"], $warehouse->id, '');
							
							// Chargement du prix d'achat
							if ($pa < 0)
								$pa = empty($cmb_product) ? $c_product->wholesale_price : $cmb_product->wholesale_price;
							
							$done = true;
							if ($qty > 0)
								$this->stockManager->addProduct(
													$product["id_product"],
													$product["id_product_attribute"],
													$warehouse,
													$qty,
													$this->stock_reason,
													$pa
												);
							else
								$this->stockManager->removeProduct(
													$product["id_product"],
													$product["id_product_attribute"],
													$warehouse,
													$qty * -1,
													$this->stock_reason,
													$pa
												);
						}
					}
					
					if ($done)
						StockAvailable::synchronize($product["id_product"]);					
				}
				else
				{
					// Stocks manuels
					$current_stock = StockAvailable::getQuantityAvailableByProduct(
											$product["id_product"],
											$product["id_product_attribute"]);
					if ($total_stock != $current_stock)
					{
						StockAvailable::updateQuantity($product["id_product"], $product["id_product_attribute"], 
							$total_stock - $current_stock);
					}
				}	
			}
		}
	}
	