<?php
	/*
	* 2018 - VEGA Informatique
	*
	* NOTICE OF LICENSE
	*
	* This source file is subject to the Open Software License (OSL 3.0)
	* that is bundled with this package in the file LICENSE.txt.
	* It is also available through the world-wide-web at this URL:
	* http://opensource.org/licenses/osl-3.0.php
	*
	* DISCLAIMER
	*
	* Do not edit or add to this file if you wish to upgrade PolarisLink to newer
	* versions in the future. 
	*
	*  @author VEGA Informatique <contact@vega-info.fr>
	*  @copyright  2018 VEGA Informatique
	*  @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
	*/

if (!defined('_PS_VERSION_'))
  exit;

require_once(_PS_ROOT_DIR_ . "/modules/polarislink/classes/PolarisLinkJobManager.php");
require_once(_PS_ROOT_DIR_ . "/modules/polarislink/classes/PolarisLinkImport_product.php");
require_once(_PS_ROOT_DIR_ . "/modules/polarislink/classes/PolarisLinkImport_manufacturer.php");
require_once(_PS_ROOT_DIR_ . "/modules/polarislink/classes/PolarisLinkImport_category.php");

class AdminPolarisLinkIPSConvertController extends ModuleAdminController
{	
	public function __construct()
	{
		$this->context = Context::getContext();			
		$this->bootstrap = true;
					
		parent::__construct();		
	}
	
	
	
	/* On procède à l'affichage */
	public function renderList()
	{		
		$output = '';
		
		try
		{
			$db = Db::getInstance();
			$useDb = true;
			
			// Pas de limite de temps !
			set_time_limit(0);
				
			$sites = $this->module->Config;
			
			$i_site = Tools::getValue('PPL_VEGA_SITE');
			$url = Tools::getValue('PPL_VEGA_IPS_URL');
					
			if (!empty($url))
			{
				$EOL = "\r\n";
				
				$site = null;
				
				foreach ($sites as $s)
				{
					if ($s->LinkID == $i_site)
					{
						$site = $s;
						break;
					}
				}
				if (empty($site))
					throw new Exception("Site PolarisLink indiqué non trouvé");
				
				$jobsManager = new PolarisLinkJobManager($site);
				
				// Tirage donnée
				if ($url[strlen($url)-1] != "/")
					$url .= "/";
				
				Logger::addLog(sprintf("Demande d'aspiration des URLs depuis le site IPS %s ...", $url), 1, 1, $this->module->displayName, 1, true);
				$output .= sprintf("<h2>Redirection du site %s</h2>", $url);
				
				if (!($data = file_get_contents($url."jsmap")))
					throw new Exception("Impossible d'importer les définitions de redirections");
				
				$data = json_decode($data, true);
				if (empty($data))
					throw new Exception("Impossible de décoder les définitions de redirections");
				
				$link = new Link();
				
				$baseUrlLength = strlen($link->getBaseLink()) -1;
				
				$missings = array();
				$htaccess = "";
				
				if ($useDb)
					$db->Execute("DELETE FROM `"._DB_PREFIX_."polarislink_ips_redir`");
				
				// Marques ...
				$manufacturersImporter = new PolarisLinkImport_manufacturer();
				$manufacturersImporter->Init($jobsManager, $site);

				$htaccess .= "# Redirection marques IPS".$EOL;
				foreach ($data["manufacturers"] as $id => $manufacturer)
				{
					$manufacturer_id = $manufacturersImporter->getManufacturerId($id, $manufacturer["name"]);
					if (!empty($manufacturer_id))
					{
						$obj = new Manufacturer($manufacturer_id);
						
						if ($useDb)
							$db->Execute("INSERT INTO `"._DB_PREFIX_."polarislink_ips_redir` (old_url, new_url) VALUES ('".pSQL($manufacturer["url"])."', '".pSQL(substr($link->getManufacturerLink($obj), $baseUrlLength))."')");
						else
							$htaccess .= sprintf("RewriteRule ^%s$ %s [L,R=301]", 
										$manufacturer["url"],
										substr($link->getManufacturerLink($obj), $baseUrlLength)).$EOL;
					}
					else
						$missings[] = $manufacturer["url"]." (marque)";					
				}
				
				// Rayons ...
				$shelvesImporter = new PolarisLinkImport_category();
				$shelvesImporter->Init($jobsManager, $site);
				
				$htaccess .= "# Redirection rayons IPS".$EOL;
				foreach ($data["shelves"] as $id => $shelve)
				{
					$shelve_id = $shelvesImporter->getCategory("ray ".$id, $shelve["name"], $site->CategoryForRFS);
					if (!empty($shelve_id))
					{
						$obj = new Category($shelve_id);
						
						$cu = substr($link->getCategoryLink($obj), $baseUrlLength);
						
						if ($useDb)
						{
							$db->Execute("INSERT INTO `"._DB_PREFIX_."polarislink_ips_redir` (old_url, new_url) VALUES ('".pSQL($shelve["url"])."', '".pSQL($cu)."')");
							foreach ($shelve["alternates"] as $alt)
								$db->Execute("INSERT INTO `"._DB_PREFIX_."polarislink_ips_redir` (old_url, new_url) VALUES ('".pSQL($alt)."', '".pSQL($cu)."')");
						}
						else
						{
							$htaccess .= sprintf("RewriteRule ^%s$ %s [L,R=301]", $shelve["url"], $cu).$EOL;
							foreach ($shelve["alternates"] as $alt)
								$htaccess .= sprintf("RewriteRule ^%s$ %s [L,R=301]", $alt, $cu).$EOL;
						}
					}
					else
						$missings[] = $shelve["url"]." (rayon)";
				}
				
				// Produits ...
				$productsImporter = new PolarisLinkImport_product();
				$productsImporter->Init($jobsManager, $site);
				
				$htaccess .= "# Redirection produits IPS".$EOL;
				foreach ($data["products"] as $id => $product)
				{
					// Identification produit
					$productFound = $productsImporter->GetProductCombinationFor(join(",", $product["codes"]));
					if (!empty($productFound))
					{
						$obj = new Product($productFound["id_product"]);
						
						if ($useDb)
						{
							$db->Execute("INSERT INTO `"._DB_PREFIX_."polarislink_ips_redir` (old_url, new_url) VALUES ('%,id-".pSQL($id)."', '".pSQL(substr($link->getProductLink($obj), $baseUrlLength))."')");
						}
						else
							$htaccess .= sprintf("RewriteRule ^(.*),id-%s$ %s [L,R=301]", 
										$id,
										substr($link->getProductLink($obj), $baseUrlLength)).$EOL;
					}
					else
						$missings[] = $product["url"]." (produit)";					
				}
				
				if (!$useDb)
				{
					$htaccessFilename = _PS_ROOT_DIR_."/.htaccess";
				
					// Sauvegarde du fichier .htaccess
					copy($htaccessFilename, $htaccessFilename.".backup");
					
					$currentContents = file_get_contents($htaccessFilename);
					
					// Nettoyage ancien passage !
					$currentContents = preg_replace("/\R# IPS \(begin\)\R.*# IPS \(end\)\R/mus", "", $currentContents);				
					
					$newContents = $currentContents.$EOL."# IPS (begin)".$EOL.$htaccess.$EOL."# IPS (end)".$EOL;
					file_put_contents($htaccessFilename, $newContents);
				}
				
				if (count($missings) > 0)
				{
					$output .= "<h3>Erreurs de redirections</h3>";
					$output .= "<p>Liste des URL non trouvées pour redirection: </p>";
					$output .= "<ul>";
					foreach ($missings as $missing)
						$output .= "<li>".$missing."</li>";
					$output .= "</ul><br>";
				}
								
				if ($useDb)
				{
					//
				}
				else
				{
					$output .= "<h3>Rapport du fichier de redirections installées</h3>";			
					$output .= "<p>Ajout des redirections dans le fichier de redirection .htaccess comme suit : </p>";
					$output .= "<pre>";
					$output .= file_get_contents($htaccessFilename);				
					$output .= "</pre>";
				}				
			}

			if (empty($url))
			{
				// Demande de l'URL !				
				$helper = new HelperForm();
			 
				// Module, token and currentIndex
				$helper->module = $this->module;
				$helper->name_controller = "AdminPolarisLinkIPSConvert";
				$helper->token = Tools::getAdminTokenLite('AdminModules');
				//$helper->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;
				 
				// Language
				$id_lang = $this->context->language->id;
				$helper->default_form_language = $id_lang;
				$helper->allow_employee_form_lang = $id_lang;
				 
				// Title and toolbar
				$helper->title = "Récupération de site IPS";
				$helper->show_toolbar = false;        // false -> remove toolbar
				$helper->toolbar_scroll = true;      // yes - > Toolbar is always visible on the top of the screen.
				$helper->submit_action = 'submit';
				
				$helper->fields_value['PPL_LINK_ACTION'] = '0';
				
				// Init Fields form array		
				$fields_form[0]['form'] = array(
					'input' => array(
								array('type' => 'hidden', 'name' => 'PPL_LINK_ACTION'),
								array(
									'type' => 'text',
									'label' => $this->l("URL du site IPS à récupérer"),							
									'name' => 'PPL_VEGA_IPS_URL',					
									'size' => 40,
									'required' => true,
								),
								array(
									'type' => 'select',
									'label' => $this->l("Site PolarisLink concerné"),
									'name' => 'PPL_VEGA_SITE',
									'required' => true,
									'options' => array(
													'query' => $sites,
													'id' => 'LinkID',
													'name' => 'LinkName',
													)
								),
						),
					'submit' => array(
						'title' => "Récupérer",
						'class' => 'btn btn-default pull-right'
					)
				);	
										  
				// Load current value
				$helper->fields_value['PPL_VEGA_SITE'] = "0";
				$helper->fields_value['PPL_VEGA_IPS_URL'] = "";
			 
				if (stripos($this->context->employee->email, "@vega-info.fr") !== false)
					$output .= $helper->generateForm($fields_form);
			}
			
			if ($useDb)
			{
				$output .= '<div class="panel">';
				$output .= '<style>';
				$output .= '.redir_tbl { width: 100%; } ';
				$output .= '.redir_tbl tr:hover { background: #ccc; } ';
				$output .= '</style>';
				$output .= "<h3>Redirections installées</h3>";			
				$output .= '<table class="redir_tbl"><thead><tr><th>Ancienne URL</th><th>Redirection vers</th></tr></thead><tbody>';
				foreach ($db->ExecuteS("SELECT * FROM `"._DB_PREFIX_."polarislink_ips_redir` ORDER BY old_url") as $row)
				{
					$output .= "<tr>";
					$output .= '<td>'.$row['old_url'].'</td><td>'.$row['new_url'].'</td>';
					$output .= "</tr>";
				}
				$output .= '</tbody></table>';
				$output .= '</div>';
			}
		}
		catch (Exception $ex)
		{
			Logger::addLog("Erreur lors de la conversion du site IPS : ".$ex->getMessage(), 2, 1, $this->module->displayName, 1, true);
			$output .= '<div class="alert alert-danger">'.Tools::displayError($ex->getMessage()).'</div>';
		}
		
		return $output;
	}
}