<?php
	/*
	* 2018 - VEGA Informatique
	*
	* NOTICE OF LICENSE
	*
	* This source file is subject to the Open Software License (OSL 3.0)
	* that is bundled with this package in the file LICENSE.txt.
	* It is also available through the world-wide-web at this URL:
	* http://opensource.org/licenses/osl-3.0.php
	*
	* DISCLAIMER
	*
	* Do not edit or add to this file if you wish to upgrade PolarisLink to newer
	* versions in the future. 
	*
	*  @author VEGA Informatique <contact@vega-info.fr>
	*  @copyright  2018 VEGA Informatique
	*  @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
	*/

	if (!defined('_PS_VERSION_'))
		exit;
			
	require_once( _PS_ROOT_DIR_ . "/modules/polarislink/classes/PolarisLinkAPI.php");
	require_once( _PS_ROOT_DIR_ . "/modules/polarislink/classes/PolarisLinkJobManager.php");
	
	/*
	 * exécution des travaux
	 *
	 */
	class PolarisLinkAPI_pullchangesModuleFrontController extends PolarisLinkAPI
	{
		protected function writeXmlElement($elementName, $value, $attrs = null)
		{
			$attrStr = "";
			if (!empty($attrs))
				foreach ($attrs as $n => $v)
					$attrStr .= " ".$n."=\"".htmlspecialchars($v, ENT_XML1, 'UTF-8')."\"";
			
			return "<".$elementName.$attrStr.">".htmlspecialchars($value, ENT_XML1, 'UTF-8')."</".$elementName.">";
		}
		
		protected function getCustomersChanges($sinceDate)
		{
			if (!$this->Link->OPT_ALLOW_E_CUSTOMERS)
				return;
			
			$genders = array();
			foreach (Gender::getGenders() as $gender)
				$genders[$gender->id] = $gender->name;
			$customers = array();
			
			$addressCache = new PolarisLinkExtRefCache($this->Link->LinkID, "address");

			// requête pour avoir les id des clients ayant changés
			foreach (Db::getInstance()->executeS(sprintf("SELECT id_customer FROM "._DB_PREFIX_."customer WHERE deleted = 0 AND date_upd > '%s'", date("Y-m-d H:i:s", $sinceDate)), true, false) as $row)
				$customers[$row["id_customer"]] = $row["id_customer"];
				
			// requête pour avoir les id des clients ayant changés
			foreach (Db::getInstance()->executeS(sprintf("SELECT c.id_customer FROM "._DB_PREFIX_."address AS a LEFT JOIN "._DB_PREFIX_."customer AS c ON (c.id_customer = a.id_customer) WHERE c.deleted = 0 AND a.id_customer > 0 AND c.id_customer IS NOT NULL AND a.date_upd > '%s'", date("Y-m-d H:i:s", $sinceDate)), true, false) as $row)
				$customers[$row["id_customer"]] = $row["id_customer"];

			foreach ($customers as $customer_id)
			{
				$customer = new Customer($customer_id);
				
				echo "<customer>";
				
				echo "<mails>";
				echo $this->writeXmlElement("mail", $customer->email);
				$altmails = array();
				foreach (Db::getInstance()->executeS(sprintf("SELECT CAST(AES_DECRYPT(UNHEX(mail), sha1('%s')) AS char) AS mail FROM "._DB_PREFIX_."polarislink_mails WHERE id_customer = %s", sha1(_COOKIE_KEY_), (int) $customer_id)) as $row)
					echo $this->writeXmlElement("mail", $row["mail"]);
				echo "</mails>";
				
				if ($customer->id_gender > 0)
					echo $this->writeXmlElement("title", $genders[$customer->id_gender]);

				echo $this->writeXmlElement("firstname", mb_strtolower($customer->firstname) == "inconnu" ? "" : $customer->firstname);
				echo $this->writeXmlElement("lastname", mb_strtolower($customer->lastname) == "inconnu" ? "" : $customer->lastname);
				
				if ( $customer->birthday > "1901-01-01" )
					echo $this->writeXmlElement("birthday", $customer->birthday);
				
				echo $this->writeXmlElement("optin", $customer->optin);
				echo $this->writeXmlElement("notes", $customer->note);
				
				echo "</customer>\n";
			}			
		}
		
		protected function getOrders($sinceDate)
		{
			if (empty($this->Link->OrderExportStateID) || count($this->Link->OrderExportStateID) <= 0 || !$this->Link->OPT_ALLOW_E_SALES)
				return;
			
			// On évite de rappatrier toutes les ventes !!
			$installDate = Configuration::get("POLARISLINK_INSTALL_DATE", "");
			if (empty($installDate))
			{
				$installDate = date("Y-m-d H:i:s");
				Configuration::updateValue("POLARISLINK_INSTALL_DATE", $installDate);
			}
			
			foreach (Db::getInstance()->executeS(sprintf("SELECT DISTINCT id_order FROM "._DB_PREFIX_."order_history WHERE date_add > '%s' AND date_add > '%s' AND id_order_state IN (%s)", 
						$installDate,
						date("Y-m-d H:i:s", $sinceDate), join(', ', $this->Link->OrderExportStateID)), true, false) as $row)
			{
				$order = new Order($row["id_order"]);
				if (Validate::isLoadedObject($order) && $order->id_currency == $this->Link->CurrencyID)
				{
					$customer = new Customer($order->id_customer);
					echo "<order id=\"".$order->id."\">";
					
					echo $this->writeXmlElement("reference", $order->reference);
					$time = strtotime($order->date_add);
					echo $this->writeXmlElement("date", date("Y-m-d H:i:s", $time - date("Z", $time)));
					echo $this->writeXmlElement("payment", $order->module);

					if ($this->Link->OPT_ALLOW_E_CUSTOMERS && Validate::isLoadedObject($customer))
						echo $this->writeXmlElement("mail", $customer->email);
										
					$discounts = $order->total_discounts_tax_excl;
					$discounts_wt = $order->total_discounts_tax_incl;
					
					$vouchers = array();
					foreach ($order->getCartRules() as $discount)
					{
						$voucher = new CartRule($discount["id_cart_rule"]);
						if (Validate::isLoadedObject($voucher))
							if (!empty($voucher->code))
							{
								$vouchers[] = array("code" => $voucher->code, "amount" => $discount["value"]);
								$discounts_wt -= $discount["value"];
								$discounts -= $discount["value_tax_excl"];
							}
					}
					
					if (count($vouchers) > 0)
					{
						echo "<vouchers>";
						foreach ($vouchers as $voucher)
							echo $this->writeXmlElement("voucher", sprintf("%.02f", $voucher["amount"]), array("code" => $voucher["code"]));
						echo "</vouchers>";
					}
					
					echo $this->writeXmlElement("total_discounts_tax_incl", sprintf("%0.2f", $discounts_wt));
					echo $this->writeXmlElement("total_discounts_tax_excl", sprintf("%0.2f", $discounts));
					echo $this->writeXmlElement("total_products_tax_incl", sprintf("%0.2f", $order->total_products_wt));
					echo $this->writeXmlElement("total_products_tax_excl", sprintf("%0.2f", $order->total_products));
					echo $this->writeXmlElement("total_shipping_tax_incl", sprintf("%0.2f", $order->total_shipping_tax_incl));
					echo $this->writeXmlElement("total_shipping_tax_excl", sprintf("%0.2f", $order->total_shipping_tax_excl));
					echo $this->writeXmlElement("total_shipping_tax_rate", sprintf("%0.2f", $order->carrier_tax_rate));
					echo $this->writeXmlElement("total_wrapping_tax_incl", sprintf("%0.2f", $order->total_wrapping_tax_incl));
					echo $this->writeXmlElement("total_wrapping_tax_excl", sprintf("%0.2f", $order->total_wrapping_tax_excl));					
					echo $this->writeXmlElement("total_wrapping_tax_rate", sprintf("%0.2f", $order->carrier_tax_rate));
					
					echo $this->writeXmlElement("total_tax_incl", sprintf("%0.2f", $order->total_products_wt + $order->total_shipping_tax_incl + $order->total_wrapping_tax_incl));
					echo $this->writeXmlElement("total_paid", sprintf("%0.2f", $order->total_paid));
					
					echo "<details>";
					foreach ($order->getProducts() as $detail)
					{
						$sku = $detail["product_supplier_reference"];
						echo "<product sku=\"".$sku."\" ean13=\"".$detail["product_ean13"]."\">";
						
						if ((int) $detail["id_warehouse"] > 0)
						{
							$warehouse = new Warehouse($detail["id_warehouse"]);
							if (Validate::isLoadedObject($warehouse))
								echo $this->writeXmlElement("warehouse", $warehouse->reference);
						}
						
						echo $this->writeXmlElement("qty", $detail["product_quantity"]);
						echo $this->writeXmlElement("tax_rate", $detail["tax_rate"]);
												
						echo $this->writeXmlElement("product_price_tax_excl", sprintf("%0.2f", $detail["original_product_price"]));
						echo $this->writeXmlElement("product_price_tax_incl", sprintf("%0.2f", $detail["original_product_price"] * (1 + ($detail["tax_rate"] / 100.0))));
						
						echo $this->writeXmlElement("unit_price_tax_excl", sprintf("%0.2f", $detail["unit_price_tax_excl"]));
						echo $this->writeXmlElement("unit_price_tax_incl", sprintf("%0.2f", $detail["unit_price_tax_incl"]));
						
						echo $this->writeXmlElement("total_price_tax_excl", sprintf("%0.2f", $detail["total_price_tax_excl"]));
						echo $this->writeXmlElement("total_price_tax_incl", sprintf("%0.2f", $detail["total_price_tax_incl"]));
						
						echo $this->writeXmlElement("reduction_percent", sprintf("%0.2f", $detail["reduction_percent"]));
						echo $this->writeXmlElement("reduction_amount_tax_excl", sprintf("%0.2f", $detail["reduction_amount_tax_excl"]));
						echo $this->writeXmlElement("reduction_amount_tax_incl",sprintf("%0.2f",  $detail["reduction_amount_tax_incl"]));
						
						echo "</product>";
					}
					echo "</details>";
					
					echo "</order>\n";
				}
			}
		}
		
		protected function runAPI()
		{		
			$since = isset($_REQUEST['since']) ? trim($_REQUEST['since']) : null;
			if (empty($since))
				throw new Exception("paramètre «since» non défini");			
			
			$since = strtotime($since);
			//// On ajoute le timeshift du serveur !!
			//$since += date("Z", $since);

			if (empty($since) || $since < 0)
				throw new Exception("paramètre «since» spécifié n'est pas une date correcte");
						
			header("Content-Type: application/xml; charset=utf-8");
			echo '<?xml version="1.0" encoding="UTF-8"?>'."\n";
			
			$serverDate = date('Y-m-d H:i:s');
			
			echo "<changes date=\"".$serverDate."\">\n";
			$this->getCustomersChanges($since);
			$this->getOrders($since);
			echo "</changes>\n";

			return true;
		}
	}
 