<?php
/*
* 2018 - VEGA Informatique
*
* NOTICE OF LICENSE
*
* This source file is subject to the Open Software License (OSL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/osl-3.0.php
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PolarisLink to newer
* versions in the future. 
*
*  @author VEGA Informatique <contact@vega-info.fr>
*  @copyright  2018 VEGA Informatique
*  @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
*/

if (!defined('_PS_VERSION_'))
  exit;

require_once("classes/PolarisLinkSite.php");  

/*
	Exemple d'utilisation du regroupement de modèles :
	A insérer dans product.tpl :
	
	{************************************************************ CHOIX COLORIS ************************************************************}
			{hook h='actionPolarisLinkLoadRelatedProducts'}
			{if count($relatedProducts) > 1}			
			<style>
				.related-products-selector {
				}
				
				.related-products-selector li {
					float: left;
					font-size: 8pt;
					margin-right: 4px;
				}
				
				.related-products-selector .active {
					background: darkorange;
				}
				
				.related-products-selector .active a {
					border: 1px solid darkorange;
				}
				
				.related-products-selector .active img {
					opacity: 0.7;
				}
				
				.related-products-selector a {
					display: block;
					border: 1px solid #ccc;
					border-radius: 24px;
					width: 48px;
					height: 48px;
					line-height: 24px;
					overflow: hidden;
					text-align: center;
					background: white;
					text-decoration: none;
				}
				
				.related-products-selector a:hover,
				.related-products-selector a:focus
				{
					border: 1px solid darkorange;
					text-decoration: none; 
					outline:none;
				}
	
			</style>
			<p style="font-weight: bold;">Existe en&nbsp;: </p>
			<ul class="related-products-selector">
			{foreach from=$relatedProducts item=r_product}
				<li><a {if $r_product->id == $product->id}class="active"{/if} href="{$r_product->getLink()}" title="{$r_product->name|capitalize|escape:'html':'UTF-8'}">
					{assign var="id_cover" value=$r_product->getCoverWs()}
					<img itemprop="image" src="{if $id_cover == ""}/img/p/fr-default-selection.jpg{else}{$link->getImageLink($r_product->link_rewrite, $id_cover, 'selection')|escape:'html':'UTF-8'}{/if}" title="{$r_product->name|capitalize|escape:'html':'UTF-8'}" alt="{$r_product->name|capitalize|escape:'html':'UTF-8'}" />
				</a></li>
			{/foreach}
			</ul>
			<div style="clear: both; margin-bottom: 2ex;"></div>
			{/if}
			{************************************************************ CHOIX COLORIS ************************************************************}	
*/

class PolarisLink extends Module
{
	public $Config = null;
	
	public function __construct()
	{
		$this->name = 'polarislink';
		$this->tab = 'administration';
		$this->version = '2.28';
		$this->bootstrap = true;
		$this->author = 'VEGA Informatique';
		$this->need_instance = 0;
		$this->ps_versions_compliancy = array('min' => '1.6', 'max' => '1.6');
		$this->dependencies = array();

		parent::__construct();
 
		$this->displayName = 'PolarisLink';
		$this->description = $this->l('Permet de synchroniser votre Prestashop et votre logiciel Polaris.');
		$this->confirmUninstall = $this->l('Etes-vous sûr de vouloir désinstaller ce module ?');
		
		if (Module::isInstalled("polarislink"))
		{
			// Chargement de la configuration
			$this->LoadConfig();
		}		
	}	
	
	public function LoadConfig($check = false)
	{
		$this->Config = PolarisLinkSite::loadConfig();
		if (empty($this->Config) || count($this->Config) == 0)
		{
			$cfg = new PolarisLinkSite();
			$cfg->save();
			
			$this->Config = array($cfg);
			$check = true;
		}
		
		if ($check)
		{
			foreach ($this->Config as $cfg)
			{
				if ($cfg->checkConfig())
					$cfg->save();
			}
		}
	}	
	
	/*
		Petite fonction bien pratique appelée pour se mettre à jour lors d'une installation.
		Cela évite de faire le boulot en double.
		Reprends tous les fichiers de mises à jour progressifs et les passes à la queue-leu-leu.
	*/
	public function upgrade_to_current()
	{
		$path = $this->local_path;
		
		$files = array();
		/* on cherche tous les upgrades */
		foreach (scandir($path."upgrade") as $file)
		{
			if (substr($file, 0, 8) == "Upgrade-")
			{
				$withoutExt = substr($file, 0, strlen($file) - 4);
				$withoutExt = substr($withoutExt, strlen("Upgrade-"));
				
				$files[$withoutExt] = $file;
			}
		}
		
		/* on va les passer dans l'ordre, version par version */
		if (!uksort($files, "version_compare"))
			$this->_errors[] = Tools::displayError("impossible de trier les versions");
		
		foreach ($files as $version => $file)
		{
			/* pour chacun d'entre eux, on passe le fichier d'upgrade */
			include ($path."upgrade/$file");

			$upgrade_func = "upgrade_module_".str_replace(".", "_", $version);			
			if (!function_exists($upgrade_func))
			{
				$this->_errors[] = Tools::displayError("impossible de trouver les méthodes d'installation de base en version $version, abandon !");
				return false;
			}
			
			if (!$upgrade_func($this))
			{
				$this->_errors[] = Tools::displayError("impossible d'installer les données de la version $version, abandon !");
				return false;
			}
		}

		$this->registerHooks();
		
		return true;
	}

	public function getHooks()
	{
		return array(
			'dashboardZoneOne',
			'actionPolarisLinkLoadRelatedProducts',
			'actionDispatcher',
			'displayProductPriceBlock',
			'actionPolarisLinkProductSelector',
			);
	}

	/*
		Réenregistrement des hooks...
	*/
	public function registerHooks()
	{
		foreach ($this->getHooks() as $hook)
			$this->registerHook($hook);
	}
	
	/*
		Procédure d'installation.
		S'occupe de mettre en place les premiers hooks, le reste étant fait par
		l'application des mises à jours successives :)
	*/
	public function install()
	{
		if (!parent::install())
			return false;
		
		try
		{
			// Install admin module ...
			$tab = new Tab();
			$tab->active = 1;
			$tab->class_name = "AdminPolarisLinkVegaAutoUpdater";
			$tab->name = array();
			foreach (Language::getLanguages(true) as $lang)
				$tab->name[$lang['id_lang']] = "Mise à jour du module PolarisLink";			
			$tab->id_parent = -1;
			$tab->module = $this->name;
			$tab->add();
			
			if (!$this->upgrade_to_current())			
				throw new Exception("Erreur lors du déploiement de la base de données");
			
			$this->LoadConfig();			
			
			/* si tout le monde est passé, alors on est content ! */		
			return true;
		}
		catch (Exception $ex)
		{
			$this->_errors[] = Tools::displayError($ex->getMessage());
			$this->uninstall();
			
			return false;
		}
	}
	
	/*
		Désinstallation: on fait le ménage
	*/
	public function uninstall()
	{	
		// Uninstall Module
		if (!parent::uninstall())
			return false;
		
		// On nettoie les modules
		foreach (Tab::getCollectionFromModule($this->name) as $tab)
			$tab->delete();
			
		// On nettoie les points d'accroche
		foreach ($this->getHooks() as $hook)
			$this->registerHook($hook);
			
		return true;	
	}

	/*
		Envoi les métriques à VEGA Informatique à des fins de maintenance et de licences
	*/
	public function sendMetrics($force = false)
	{
		try
		{
			$lastMetricsSent = Configuration::get("POLARISLINK_METRICS_SENT");		
			if ($force || $lastMetricsSent < date("Y-m-d") )
			{
				require_once(_PS_ROOT_DIR_ . "/modules/polarislink/classes/PolarisLinkJobManager.php");
				require_once(_PS_ROOT_DIR_ . "/modules/polarislink/classes/VegaDcc.php");

				$data = PolarisLinkJobManager::getGlobalProgress();
				$dcc = new VegaDcc("polarislink", "j3Sg4b96KJfAWpW3ahK5w8A2");

				$m = [ 
					"ps_version" => _PS_VERSION_,
					"version" => $this->version,
					"url" => _PS_BASE_URL_.__PS_BASE_URI__,
					"errors" => $data["err"],
				];

				if ($data && isset($data["err"]))
					$m["errors"] = $data["err"];

				$dcc->sendMetrics($m);				
			}
		}
		catch (\Exception $ex)
		{
			// Pas grave
			//echo $ex->getMessage();
		}

		// Métriques envoyées, même s'il y a eu une erreur !
		Configuration::updateValue("POLARISLINK_METRICS_SENT", date("Y-m-d"));
	}
	
	/*
		Signale une erreur à afficher en Web2.0 dans l'interface
		lors de son rendu.
		Ne déclenche pas l'affichage immédiat de l'erreur mais simplement programme
		son rendu.
		UNIQUEMENT valable lors de l'installation/initialisation !
	*/
	public function install_error($msg)
	{
		$this->_errors[] = $msg;
	}	
	
	private function getMultipleParams($paramId, $possibleChoices, $possibleChoiceKey)
	{
		$checked = array();
		foreach ($possibleChoices as $choice)
		{
			if (Tools::getValue($paramId.'_'.$choice[$possibleChoiceKey]))
				$checked[] = $choice[$possibleChoiceKey];
		}
		return $checked;
	}
	
	public function getAuthTokenStr()
	{
		$tk = "PolarisLinkAPI;".$_SERVER["REMOTE_ADDR"].";";
		return Tools::getAdminToken($tk);
	}
	
	public function hookActionDispatcher($params)
	{
		if ($params["controller_class"] != "PageNotFoundController")
			return false;
								
		$request = $_SERVER["REQUEST_URI"];
		
		$db = Db::getInstance(_PS_USE_SQL_SLAVE_);
		if ($results = $db->ExecuteS("SELECT new_url FROM `"._DB_PREFIX_."polarislink_ips_redir` WHERE '".pSQL($request)."' LIKE old_url"))
		{
			foreach ($results as $result)
			{
				if (!empty($result["new_url"]))
				{
					// Ok, redirection !
					$to = Tools::getShopProtocol().Tools::getShopDomainSsl().$result["new_url"];
					
					header("Location: ".$to, true, 301); 
					header("Content-Type: text/html; charset=utf8");
					echo '<html><head></head><body>Vous allez être redirigé vers la page <a href="'.$to.'">'.$to.'</a>.</body></html>';
					
					die();
				}
			}
		}
		
		return false;
	}

	/*
		Construction du sélecteur visuel
	*/
	public function buildVisualProductsSelector($id_product)
	{
		// Chargement des produits liés ...
		$context = Context::getContext();		

		$excludeCurrent = Configuration::get("POLARISLINK_PRODUCTS_SELECTOR_EXCLUDE_CURRENT");
		$onlyWithStocks = Configuration::get("POLARISLINK_PRODUCTS_SELECTOR_STOCK");
		
		$results = $this->loadRelatedProductsInfo(array($id_product));

		$products = array();
		foreach ($results["products"][$id_product] as $product)
		{
			if (!$product->active)
				continue;

			if ($onlyWithStocks && $product->quantity <= 0)
				continue;

			if ($excludeCurrent && $product->id == $id_product)
				continue;

			$products[] = $product;
		}

		if (count($products) > 0)
		{
			// Identification du produit par défaut
			if (Configuration::get("POLARISLINK_PRODUCTS_SELECTOR_CANONICAL"))
			{
				$current_product = null;
				$canonical_product = null;
				foreach ($products as $product)
				{
					if ($canonical_product == null || $canonical_product->id > $product->id)
						$canonical_product = $product;
					if ($product->id == $id_product)
						$current_product = $product;
				}

				if ($canonical_product->id == $id_product ||
					($current_product && trim($canonical_product->description) != trim($current_product->description)))
					$canonical_product = null;
			}

			// Si ce n'est pas le produit en cours ...
			if ($canonical_product)
				header( sprintf('Link: <%s>; rel="canonical"', $context->link->getProductLink($canonical_product, null, null, null, null, null, Product::getDefaultAttribute($canonical_product->id))) );	

			echo '<style>';
			echo Configuration::get("POLARISLINK_PRODUCTS_SELECTOR_CSS");
			echo '</style>';

			echo '<div class="polarislink-critselectblock">';
			//echo '<p class="polarislink-icien"><span class="lbl">'.Translate::getModuleTranslation("polarislink", "Présenté ici en :", "").'</span><span class="vle"></span></p>';
			echo '<p class="polarislink-existeen"><span class="lbl">'.
				($excludeCurrent ? 	Translate::getModuleTranslation("polarislink", "Existe également en :", "") : 
									Translate::getModuleTranslation("polarislink", "Existe en :", "")).'</span></p>';
			echo '<ul class="polarislink-selecteur">';

			foreach ($products as $product)
			{
				$product_title = htmlspecialchars($product->name);
				echo '<li>';
				echo '<a '.($product->id == $id_product ? 'class="active"' : '').
						'href="'.$context->link->getProductLink($product, null, null, null, null, null, Product::getDefaultAttribute($product->id)).'" '.
						'title="'.$product_title.'">';

				$cover = $product->getCover($product->id);
				if (!empty($cover) && isset($cover['id_image']) && !empty($cover['id_image']))
					$img_url = $context->link->getImageLink($product->link_rewrite, $cover['id_image'], 'selection');
				else
					$img_url = "/img/p/fr-default-selection.jpg";
				echo '<img itemprop="image" src="'.$img_url.'" title="'.$product_title.'" alt="'.$product_title.'" />';				
				echo '</a>';
				echo '</li>';
			}

			echo '</ul>';
			echo '<div class="polarislink-selecteur-end"></div>';
			echo '</div>';
		}
	}

	public function hookDisplayProductPriceBlock($params)
	{
		if (isset($params["type"]) && $params["type"] != "after_price")
			return;

		if (!Configuration::get("POLARISLINK_PRODUCTS_SELECTOR_USE"))
			return;	// Génération non désirée

		$this->buildVisualProductsSelector($params["product"]->id);
	}

	public function hookActionPolarisLinkProductSelector($params)
	{
		$this->buildVisualProductsSelector($params["product_id"]);
	}
	
	public function hookDashboardZoneOne($params)
	{
		$adminAddon = '';
		if (stripos($this->context->employee->email, "@vega-info.fr") !== false)
			$adminAddon = '<p>[&nbsp;<a href="'.$this->context->link->getAdminLink("AdminModules") .  
				"&configure=polarislink&tab_module=administration&module_name=polarislink".'">Configuration de la passerelle</a>&nbsp;]</p>';

		$output = '<div class="panel">
						<h3>Synchronisation Polaris v'.$this->version.'</h3>
						<p id="plink-loading" style="text-align: center;">
							<i class="icon icon-refresh icon-spin"></i>&nbsp;&nbsp;Interrogation serveur en cours...
						</p>
						<p id="plink-in-progress" style="display: none;">
							Le traitement du fichier de synchronisation est en cours...
						</p>
						<p id="plink-stalled" style="display: none;">
							Le traitement du fichier de synchronisation est arrêté&nbsp;!?
							[&nbsp;<a href="javascript: polarisLinkDoJobs();">Reprendre</a>&nbsp;]
							[&nbsp;<a href="javascript: polarisLinkClearJob();">Supprimer</a>&nbsp;]
						</p>
						<p id="plink-err" style="display: none;">
						</p>
						<p id="plink-finished" style="display: none;">
							Aucun traitement en cours.<br>							
						</p>
						<div id="plink-infos" style="display: none;">
							<div id="plink-cprogress" class="progress progress-striped active">
								<div id="plink-progress" class="progress-bar progress-bar-success" role="progressbar" aria-valuenow="60" aria-valuemin="0" aria-valuemax="100" style="width: 0%;">
								</div>
							</div>
							<ul>
								<li><span id="plink-stats">?</span> éléments traités</li>
								<li><span id="plink-errors">?</span> erreurs</li>
							</ul>
						</div>
						'.$adminAddon.'
					</div>
					'.
					"<script type=\"text/javascript\">
						function polarisLinkDoJobs()
							{
								$.ajax({
									  url: '".$this->context->link->getModuleLink('polarislink','api_run', array('pl_auth_tk' => $this->getAuthTokenStr()))."',
									}).complete(function(data) {										
										$('#plink-infos').hide();
										$('#plink-in-progress').hide();
										$('#plink-stalled').hide();
										$('#plink-finished').hide();
										$('#plink-loading').show();
										$('#plink-err').hide();
										polarisLinkRefresh(true);
									});
						}
						
						function polarisLinkClearJob()
						{
								$.ajax({
									  url: '".$this->context->link->getModuleLink('polarislink','api_downloadJob', array('pl_auth_tk' => $this->getAuthTokenStr(), 'clear' => '1'))."',
									}).complete(function(data) {										
										$('#plink-infos').hide();
										$('#plink-in-progress').hide();
										$('#plink-stalled').hide();
										$('#plink-finished').hide();
										$('#plink-loading').show();
										$('#plink-err').hide();
										polarisLinkRefresh(true);
									});
						}
						
						function polarisLinkDismissErrors()
							{
								$.ajax({
									  url: '".$this->context->link->getModuleLink('polarislink','api_checkprogress', array('pl_auth_tk' => $this->getAuthTokenStr(), 'dismiss_errors' => "1"))."',
									}).complete(function(data) {										
										$('#plink-infos').hide();
										$('#plink-in-progress').hide();
										$('#plink-stalled').hide();
										$('#plink-finished').hide();
										$('#plink-loading').show();
										$('#plink-err').hide();
										polarisLinkRefresh(true);
									});
						}
						
						function polarisLinkRefresh(do_not_recall)
						{
							$.ajax({
								  url: '".$this->context->link->getModuleLink('polarislink','api_checkprogress', array('all' => '1', 'pl_auth_tk' => $this->getAuthTokenStr()))."',
								}).complete(function(data, textStatus) {
								  
								  $('#plink-loading').hide();
								  if (data.responseJSON != undefined)
								  {
									  var pdone = Math.round(data.responseJSON.done * 100 / data.responseJSON.todo);

									  $('#plink-stats').html(data.responseJSON.done + '/' + data.responseJSON.todo);
									  $('#plink-errors').html(data.responseJSON.err);
									  
									  $('#plink-progress').html(pdone + '&nbsp;%');
									  $('#plink-progress').css('width', pdone + '%');
									  
									  $('#plink-err').hide();									  
									  
									  if (data.responseJSON.todo == 0 || data.responseJSON.done == data.responseJSON.todo)
									  {
										var finishedText = 'Aucun traitement en cours.<br>';
										if (data.responseJSON.last_sync != null)
										{
											var dt = new Date(data.responseJSON.last_sync);											
											finishedText += 'Dernière synchronisation le ' + dt.toLocaleDateString() + ' à ' + dt.toLocaleTimeString() + '.<br>';
										}
										if (data.responseJSON.err > 0)
										{
											finishedText += '<br><div class=\"alert alert-danger\"><strong>Il s\'est produit ' + data.responseJSON.err + ' erreurs</strong> lors des dernières synchronisations : <br>' +
															'[&nbsp;<a href=\"".$this->context->link->getAdminLink("AdminLogs", array("logOrderby" => "id_log", "logOrderway" => "desc"))."\"><i class=\"icon icon-question-circle\"></i> voir</a>&nbsp;] — ' +
															'[&nbsp;<a href=\"javascript: polarisLinkDismissErrors();\"><i class=\"icon icon-ok-sign\"></i> marquer comme vu</a>&nbsp;]</div>';
										}									
											
										$('#plink-infos').hide();
										$('#plink-in-progress').hide();
										$('#plink-stalled').hide();
										$('#plink-finished').html(finishedText);
										$('#plink-finished').show();
									  }
									  else
									  {
										  $('#plink-infos').show();
										  
										  $('#plink-finished').hide();
										  if (data.responseJSON.working)
										  {
											$('#plink-cprogress').addClass('progress-striped');
											$('#plink-cprogress').addClass('active');
											$('#plink-progress').addClass('progress-bar-success');
											$('#plink-progress').removeClass('progress-bar-warning');
											$('#plink-in-progress').show();
											$('#plink-stalled').hide();
										  }
										  else
										  {
											$('#plink-cprogress').removeClass('progress-striped');
											$('#plink-cprogress').removeClass('active');
											$('#plink-progress').removeClass('progress-bar-success');
											$('#plink-progress').addClass('progress-bar-warning');
											
											$('#plink-in-progress').hide();
											$('#plink-stalled').show();										  
										  }
									  }
								  }
								  else
								  {
									  $('#plink-err').html(data.statusText + ' : ' + data.responseText);
									  $('#plink-err').show();
								  }
								  
								  if (do_not_recall === undefined || !do_not_recall)
									window.setTimeout(polarisLinkRefresh, 5000);
								});
						}
						
						polarisLinkRefresh();
					</script>";
		return $output;
	}	
	
	/*
		Configuration Page
	*/
	public function getContent()
	{
		$this->LoadConfig(true);
		
		$output = "<script type=\"text/javascript\">
			/* Code generator for Affiliation and vouchers */
			function VEGAGenKey(eltName, size)
			{
					getE(eltName).value = '';
					/* There are no O/0 in the codes in order to avoid confusion */
					var chars = '123456789ABCDEFGHIJKLMNPQRSTUVWXYZ';
					for (var i = 1; i <= size; ++i)
							getE(eltName).value += chars.charAt(Math.floor(Math.random() * chars.length));
			}
		</script>";
				
		//$id_lang = Configuration::get('PS_LANG_DEFAULT');
		$id_lang = $this->context->language->id;
		
		/* Options */
		$opts = array(
						"OPT_ACTIVE_PUBLISHED" => array(
														"t" => "Active les produits publiés",
														"d" => "Cette option permet d'activer les produits publiés automatiquement",
													),
						"OPT_DELETE_WHEN_UNPUBLISHED" => array(
														"t" => "Supprimer les produits dépubliés",
														"d" => "Cette option permet de supprimer les produits quand ils sont dépubliés de Polaris plutôt que de les marquer comme désactivés",
													),
						"OPT_ALLOW_I_PHOTOS" => array(
														"t" => "Permettre l'importation des photos",
														"d" => "Si cette option est désactivée les photos transmises seront ignorées",
													),
						"OPT_ALLOW_I_NAME" => array(
														"t" => "Permettre la modification des noms des produits",
														"d" => "Si cette option est désactivée les noms transmis seront ignorées",
						),
						"OPT_ALLOW_I_MDESCRIPTIONS" => array(
														"t" => "Permettre la modification des descriptions méta des entités",
														"d" => "Si cette option est désactivée les descriptions transmises seront ignorées",
						),
						"OPT_ALLOW_I_DESCRIPTIONS" => array(
														"t" => "Permettre la modification des descriptions des entités",
														"d" => "Si cette option est désactivée les descriptions transmises seront ignorées",
													),
													
						"OPT_ALLOW_I_CATALOG" => array(
														"t" => "Permettre l'importation du catalogue",
														"d" => "Si cette option est désactivée, les modifications du catalogue produits seront ignorées",
													),
						"OPT_ALLOW_I_CUSTOMERS" => array(
														"t" => "Permettre l'importation des clients",
														"d" => "Si cette option est désactivée, les importations de clients seront ignorées",
													),
						"OPT_ALLOW_E_CUSTOMERS" => array(
														"t" => "Permettre l'exportation des clients",
														"d" => "Si cette option est désactivée, les requêtes de données sur les clients seront rejetées",
													),						
						"OPT_ALLOW_E_SALES" => array(
														"t" => "Permettre l'exportation des ventes",
														"d" => "Si cette option est désactivée, les requêtes de données sur les ventes seront rejetées",
													),
						"OPT_ALLOW_I_STOCKS" => array(
														"t" => "Permettre la mise à jour du stock",
														"d" => "Si cette option est désactivée, les tentatives de mise à jour du stock seront ignorées",
													),
						"OPT_ALLOW_I_SALE_PRICES" => array(
														"t" => "Permettre la modification des prix de vente",
														"d" => "Si cette option est désactivée, les tentatives de modification des prix de vente seront ignorées",
													),
						"OPT_ALLOW_I_PROMOS" => array(
														"t" => "Permettre l'importation des promotions",
														"d" => "Si cette option est désactivée, les tentatives d'importation des prix promotionnels seront ignorées",
													),
						"OPT_ALLOW_I_VOUCHERS_ASSETS" => array(
							"t" => "Permettre l'importation des bons d'avoir",
							"d" => "Si cette option est désactivée, les importations de bons de ce type seront ignorées",
						),
						"OPT_ALLOW_I_VOUCHERS_OFFERS" => array(
							"t" => "Permettre l'importation des bons cadeaux et de remises",
							"d" => "Si cette option est désactivée, les importations de bons de ce type seront ignorées",
						),
													
						"OPT_USE_ADV_STOCK" => array(
														"t" => "Utiliser le stock avançé si disponible",
														"d" => "Utiliser la gestion des stocks avançée pour les produits gérés par cette passerelle si elle est activée",
													),
						"OPT_USE_EAN_STOCK" => array(
														"t" => "Permettre la maj des stocks par EAN fournisseur",
														"d" => "Permet la mise à jour des stocks à partir des codes barres fournisseur (GENCODE), même si ce ne sont pas des produits gérés par Polaris",
													),
						"OPT_USE_ARCHIVES" => array(
														"t" => "Archivage des importations",
														"d" => "Cette option permet de garder les fichiers soumis pour l'importation des données plutôt que de les supprimer",
													),
													
					);
 
		$suppliers = Supplier::getSuppliers();
				
		$shops = array();
		$shopGroups = array();
		foreach (ShopGroup::getShopGroups() as $group)
			$shopGroups[$group->id] = array('name' => $group->name);
			
		foreach (Shop::getShops() as $shop)
		{
			if (!isset($shopGroups[$shop["id_shop_group"]]["shops"]))
				$shopGroups[$shop["id_shop_group"]]["shops"] = array();
			$shopGroups[$shop["id_shop_group"]]["shops"][] = $shop;
		}
		
		foreach ($shopGroups as $id_group => $group)
		{
			$shops[] = array('id_shop' => '*'.$id_group, 'name' => $group['name']);
			foreach ($group["shops"] as $shop)
				$shops[] = array('id_shop' => $shop["id_shop"], 'name' => '    '.$shop["name"]);
		}
		
		// On évite de rappatrier toutes les ventes !!
		$installDate = Configuration::get("POLARISLINK_INSTALL_DATE", "");
		if (empty($installDate))
		{
			$installDate = date("Y-m-d H:i:s");
			Configuration::updateValue("POLARISLINK_INSTALL_DATE", $installDate);
		}

		if (Tools::isSubmit('submit'.$this->name))
		{
			$action = Tools::getValue('PPL_VEGA_ACTION');
			
			// Ok étape 1, on sauve
			$reload = false;
			
			Configuration::updateValue("POLARISLINK_CRON_DATAUPDATE", Tools::getValue("POLARISLINK_CRON_DATAUPDATE"));

			Configuration::updateValue("POLARISLINK_ALLOW_MAINTENANCE_ACCESS", Tools::getValue("POLARISLINK_ALLOW_MAINTENANCE_ACCESS"));
			if ($installDate != Tools::getValue("POLARISLINK_INSTALL_DATE"))
				Configuration::updateValue("POLARISLINK_INSTALL_DATE", Tools::getValue("POLARISLINK_INSTALL_DATE"));

			Configuration::updateValue("POLARISLINK_PRODUCTS_SELECTOR_USE", Tools::getValue("POLARISLINK_PRODUCTS_SELECTOR_USE"));
			Configuration::updateValue("POLARISLINK_PRODUCTS_SELECTOR_STOCK", Tools::getValue("POLARISLINK_PRODUCTS_SELECTOR_STOCK"));
			Configuration::updateValue("POLARISLINK_PRODUCTS_SELECTOR_EXCLUDE_CURRENT", Tools::getValue("POLARISLINK_PRODUCTS_SELECTOR_EXCLUDE_CURRENT"));			
			Configuration::updateValue("POLARISLINK_PRODUCTS_SELECTOR_CSS", Tools::getValue("POLARISLINK_PRODUCTS_SELECTOR_CSS"));
			Configuration::updateValue("POLARISLINK_PRODUCTS_SELECTOR_CANONICAL", Tools::getValue("POLARISLINK_PRODUCTS_SELECTOR_CANONICAL"));

			if (empty(Configuration::get("POLARISLINK_PRODUCTS_SELECTOR_CSS")))
			{
				Configuration::updateValue("POLARISLINK_PRODUCTS_SELECTOR_CSS", 
				".polarislink-selecteur li {
	float: left;
	font-size: 8pt;
	margin-right: 4px;
}

.polarislink-critselectblock .polarislink-existeen .lbl, .polarislink-critselectblock .polarislink-icien .vle {
	font-weight: bold;
}

.polarislink-selecteur-end {
	clear: both;
}

.polarislink-selecteur .active {
	background: darkorange;
}

.polarislink-selecteur .active a {
	border: 1px solid darkorange;
}

.polarislink-selecteur .active img {
	opacity: 0.7;
}

.polarislink-selecteur a {
	display: block;
	border: 1px solid #ccc;
	border-radius: 24px;
	width: 48px;
	height: 48px;
	line-height: 24px;
	overflow: hidden;
	text-align: center;
	background: white;
	text-decoration: none;
}

.polarislink-selecteur a:hover,
.polarislink-selecteur a:focus
{
	border: 1px solid darkorange;
	text-decoration: none; 
	outline:none;
}
");
			}
			
			$index = 0;
			do
			{
				$index++;
				$linkId = intval(Tools::getValue('PPL_VEGA_'.$index.'_LINKID'));
				if (!empty($linkId))
				{
					// on retrouve la bonne configuration ...
					$cfg = null;
					foreach ($this->Config as $sk_cfg)
					{
						if ($sk_cfg->LinkID == $linkId)
						{
							$cfg = $sk_cfg;
							break;
						}
					}
					
					if ($cfg == null)
						continue;
										
					// Ok, étape 1 : on sauve ce qui doit l'être
					$baseCfgName = "PPL_VEGA_".$index."_";
					if (intval(Tools::getValue($baseCfgName.'DELETE')) > 0)
					{
						// Suppression lien
						Logger::addLog("Chg. config. : suppression lien", 1, 1, "PolarisLink", $cfg->LinkID, true);
						$cfg->delete();
						$reload = true;
					}
					else
					{
						$value = strval(Tools::getValue($baseCfgName.'ACCESSNAME'));
						if (!$value || empty($value) || !Validate::isGenericName($value))
							$output .= $this->displayError($this->l("Le nom spécifié n'est pas valide"));
						else
							$cfg->change("LinkName", $value);
						
						$value = strval(Tools::getValue($baseCfgName.'ACCESSKEY'));
						// Génération automatique ?
						if (!$value || empty($value) || !Validate::isGenericName($value) || strlen($value) < 32)
							$output .= $this->displayError($this->l("La clé fournie n'est pas valide : elle doit faire au moins 32 caractères !"));
						else
							$cfg->change("AccessKey", $value);
							
						$cfg->change("AccessActive", intval(Tools::getValue($baseCfgName.'ACCESSOK')));
						$cfg->change("OPT_SIZE_GRID", intval(Tools::getValue($baseCfgName.'SIZE_GRID')));
						
						$cfg->change("LangID", intval(Tools::getValue($baseCfgName.'LANG_ID')));
						$cfg->change("CurrencyID", intval(Tools::getValue($baseCfgName.'CURRENCY_ID')));
						
						$cfg->change("CustomersGroupID", intval(Tools::getValue($baseCfgName.'CUSTOMERSGROUP_ID')));
						$cfg->change("SupplierID", intval(Tools::getValue($baseCfgName.'SUPPLIER_ID')));
						$cfg->change("SupplierBackupID", intval(Tools::getValue($baseCfgName.'BKSUPPLIER_ID')));
						
						$cfg->change("CategoryForRFS", intval(Tools::getValue($baseCfgName.'RFS')));			
						$cfg->change("CategoryForSeason", intval(Tools::getValue($baseCfgName.'SEASON')));
						$cfg->change("CategoryForCollection", intval(Tools::getValue($baseCfgName.'COLLECTION')));
						$cfg->change("CategoryForCategory1", intval(Tools::getValue($baseCfgName.'CATEGORY1')));
						$cfg->change("CategoryForCategory2", intval(Tools::getValue($baseCfgName.'CATEGORY2')));
						$cfg->change("CategoryForCategory3", intval(Tools::getValue($baseCfgName.'CATEGORY3')));
						$cfg->change("CategoryForCategory4", intval(Tools::getValue($baseCfgName.'CATEGORY4')));
						$cfg->change("CategoryForTags", intval(Tools::getValue($baseCfgName.'TAGS')));			
												
						$cfg->change("DefaultCategoryRoot", intval(Tools::getValue($baseCfgName.'DEFAULT')));			
						$cfg->change("AutoCategory", intval(Tools::getValue($baseCfgName.'AUTOCATEGORY')));
						$cfg->change("SortingCategoryID", intval(Tools::getValue($baseCfgName.'SORTINGCATEGORYID')));
						
						
						$cfg->change("AttrGroupForAttr0", intval(Tools::getValue($baseCfgName.'ATTR_FOR_LV0')));
						$cfg->change("AttrGroupForAttr1", intval(Tools::getValue($baseCfgName.'ATTR_FOR_LV1')));
						$cfg->change("AttrGroupForAttr2", intval(Tools::getValue($baseCfgName.'ATTR_FOR_LV2')));
						$cfg->change("AttrGroupForAttr3", intval(Tools::getValue($baseCfgName.'ATTR_FOR_LV3')));
						$cfg->change("AttrGroupForAttr4", intval(Tools::getValue($baseCfgName.'ATTR_FOR_LV4')));
						
						$cfg->change("AttrGroupForAttrG1", intval(Tools::getValue($baseCfgName.'ATTR_FOR_LVG1')));
						$cfg->change("AttrGroupForAttrG2", intval(Tools::getValue($baseCfgName.'ATTR_FOR_LVG2')));
						$cfg->change("AttrGroupForAttrG3", intval(Tools::getValue($baseCfgName.'ATTR_FOR_LVG3')));
						$cfg->change("AttrGroupForAttrG4", intval(Tools::getValue($baseCfgName.'ATTR_FOR_LVG4')));

						$cfg->change("FeatureForAttr1", intval(Tools::getValue($baseCfgName.'FEATURE_FOR_LV1')));
						$cfg->change("FeatureForAttr2", intval(Tools::getValue($baseCfgName.'FEATURE_FOR_LV2')));
						$cfg->change("FeatureForAttr3", intval(Tools::getValue($baseCfgName.'FEATURE_FOR_LV3')));
						$cfg->change("FeatureForAttr4", intval(Tools::getValue($baseCfgName.'FEATURE_FOR_LV4')));
						
						$cfg->change("FeatureForAttrG1", intval(Tools::getValue($baseCfgName.'FEATURE_FOR_LVG1')));
						$cfg->change("FeatureForAttrG2", intval(Tools::getValue($baseCfgName.'FEATURE_FOR_LVG2')));
						$cfg->change("FeatureForAttrG3", intval(Tools::getValue($baseCfgName.'FEATURE_FOR_LVG3')));
						$cfg->change("FeatureForAttrG4", intval(Tools::getValue($baseCfgName.'FEATURE_FOR_LVG4')));

						$cfg->change("MethodTags", intval(Tools::getValue($baseCfgName.'TAGS_METHOD')));

						$cfg->change("OrderExportStateID", $this->getMultipleParams($baseCfgName.'EXPORT_SALES_STATUSES', OrderState::getOrderStates($id_lang), 'id_order_state'));
						$cfg->change("CatalogShopID", Tools::getValue($baseCfgName.'CATALOG_SHOP_ID'));
						$cfg->change("PricesShopID", Tools::getValue($baseCfgName.'PRICES_SHOP_ID'));
						
						foreach ($opts as $opt_name => $opt)
							$cfg->change($opt_name, intval(Tools::getValue($baseCfgName.''.$opt_name)));
							
						$cfg->save();
					}

				}
			} while (!empty($linkId));
			
			// Enfin, les actions ...
			switch (intval(Tools::getValue('PPL_LINK_ACTION')))
			{
				case 1:
					// Nouveau lien
					$cfg = new PolarisLinkSite();
					$cfg->save();
					Logger::addLog("Chg. config. : nouveau lien", 1, 1, "PolarisLink", $cfg->LinkID, true);
					$this->Config[] = $cfg;
					break;
			}
			
			if ($reload)
				$this->LoadConfig(false);
						
			if (empty($output))
				$output .= $this->displayConfirmation($this->l('Vos paramètres ont tous été mis à jour avec succès'));			
		}
		
		$list_attrs = array(
							"query" => AttributeGroup::getAttributesGroups($id_lang),
							"id" => "id_attribute_group",
							"name" => "name"
						);

		$list_nullable_attrs = array(
			"query" => [["id_attribute_group" => 0, "name" => "— Ne pas utiliser —"]],
			"id" => "id_attribute_group",
			"name" => "name"
		);

		foreach ($list_attrs["query"] as $v)
			$list_nullable_attrs["query"][] = $v;
						
		$list_categories = array(
							"query" => Category::getAllCategoriesName(null, false, false),
							"id" => "id_category",
							"name" => "name"
						); 	

		$list_features = array(
			"query" => Feature::getFeatures($id_lang),
			"id" => "id_feature",
			"name" => "name"
		); 	

		$list_nullable_features = array(
			"query" => [["id_feature" => 0, "name" => "— Ne pas utiliser —"]],
			"id" => "id_feature",
			"name" => "name"
		);		

		foreach ($list_features["query"] as $v)
			$list_nullable_features["query"][] = $v;
		
		
		$helper = new HelperForm();
			 
		// Module, token and currentIndex
		$helper->module = $this;
		$helper->name_controller = $this->name;
		$helper->token = Tools::getAdminTokenLite('AdminModules');
		$helper->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;
		 
		// Language
		$helper->default_form_language = $id_lang;
		$helper->allow_employee_form_lang = $id_lang;
		 
		// Title and toolbar
		$helper->title = $this->displayName;
		$helper->show_toolbar = false;        // false -> remove toolbar
		$helper->toolbar_scroll = true;      // yes - > Toolbar is always visible on the top of the screen.
		$helper->submit_action = 'submit'.$this->name;
		
		$helper->fields_value['PPL_LINK_ACTION'] = '0';
		
		$buttons = array(
						'add-link' => array(
							'title' => "Ajouter un lien",
							'name' => 'addlink',
							'type' => 'button',
							'class' => 'btn btn-default pull-left',
							'icon' => 'process-icon-plus',										
							'js' => "$('#PPL_LINK_ACTION').val(1); $('#configuration_form').submit();"
						  )
					 );
							 		
		$buttons['ips-convert'] = array(
									'title' => "Redirections IPS",
									'name' => 'convert',
									'type' => 'button',
									'class' => 'btn btn-default pull-left mr-2',
									'icon' => 'process-icon-flag',
									'js' => "window.location='".$this->context->link->getAdminLink('AdminPolarisLinkIPSConvert')."'"
								);

		$buttons['archive'] = array(
			'title' => "Archives",
			'name' => 'archives',
			'type' => 'button',
			'class' => 'btn btn-default pull-left mr-2',
			'icon' => 'process-icon-preview',
			'js' => "window.location='".$this->context->link->getAdminLink('AdminPolarisLinkArchive')."'"
		);
		
		$buttons['upgrade'] = array(
									'title' => "Chercher les mises à jour",
									'name' => 'upgrade',
									'type' => 'button',
									'class' => 'btn btn-default pull-left mr-2',
									'icon' => 'process-icon-update',
									'js' => "window.location='".$this->context->link->getAdminLink('AdminPolarisLinkVegaAutoUpdater')."'"
							  );
		
		// Init Fields form array		
		$fields_form[0]['form'] = array(
			'input' => array(
						array('type' => 'hidden', 'name' => 'PPL_LINK_ACTION'),						
						array(
							'type' => 'html',
							'name' => 'html_lbl',
							'label' => $this->l("Version de la passerelle"),
							'html_content' => "<h4>".$this->version."</h4>"),
						
						array(
							'type' => 'switch',
							'label' => $this->l("Permettre la connexion de VEGA Informatique à votre Backoffice Prestashop"),
							'name' => "POLARISLINK_ALLOW_MAINTENANCE_ACCESS",
							'class' => 't', 'is_bool' => true,
							'values' => array(
											array('id' => 'active_on', 'value' => 1),
											array('id' => 'active_off', 'value' => 0, 'label' => $this->l('Désactivé')),
										)
						),

						array(
							'type' => 'date',
							'label' => $this->l("Ne pas exporter les ventes antérieures au :"),
							'name' => "POLARISLINK_INSTALL_DATE",
						),	
			),
			'buttons' => $buttons,
			'submit' => array(
				'title' => $this->l('Save'),
				'class' => 'btn btn-default pull-right'
			)
		);	

		$fields_form[0]['form']['input'][] = array(
												'type' => 'switch',
												'label' => $this->l("Mise à jour des données par tâche CRON"),
												'hint' => $this->l("Met à jour les données par tâche CRON plutôt que par appel PHP externe. Cette méthode nécessite de pouvoir accéder à votre serveur d'hébergement."),
												'name' => "POLARISLINK_CRON_DATAUPDATE",
												'class' => 't', 'is_bool' => true,
												'values' => array(
																array('id' => 'active_on', 'value' => 1),
																array('id' => 'active_off', 'value' => 0, 'label' => $this->l('Désactivé')),
															)
												);

		if (Configuration::get("POLARISLINK_CRON_DATAUPDATE"))
		{
			$fields_form[0]['form']['input'][] = array(
				'type' => 'html',
				'name' => 'html_lbl',
				'html_content' => "<h4>Mise à jour des données par tâche CRON</h4>
									<p>
									Éditez votre configuration CRON (<code>crontab -e</code>) pour ajouter la ligne suivante :									
<pre>* * * * * /usr/bin/php "._PS_MODULE_DIR_."polarislink/utils/run_jobs.php --cfg "._PS_CONFIG_DIR_."
</pre></p><p><sup>NOTES : </sup> Si vous avez plusieurs versions PHP ou si votre environnement est en <i>root-jail</i> ou <i>root-cage</i>, assurez-vous que les chemins sont corrects.</p>",
				);				
		}
		
		$configPage = '';
		$cfg_index = 0;
		
		foreach ($this->Config as $cfg)
		{
			$cfg_index++;
			
			$baseCfg = 'PPL_VEGA_'.$cfg_index.'_';
						
			// Init Fields form array		
			$fields_form[$cfg_index]['form'] = array(
				'input' => array(
					array(
						'type' => 'hidden',
						'name' => $baseCfg.'LINKID'
					),
					array(
						'type' => 'hidden',
						'name' => $baseCfg.'DELETE'
					),
					array(
						'type' => 'html',
						'name' => 'html_lbl',
						'html_content' => "<h4>Accès</h4>"
					),
					array(
						'type' => 'text',
						'label' => $this->l("Nom"),
						'hint' => $this->l("Ce nom est purement indicatif. Il sert simplement à reconnaître rapidement ce paramétrage."),
						'name' => $baseCfg.'ACCESSNAME',					
						'size' => 40,
						'required' => true,
					),					
					array(
						'type' => 'textbutton',
						'label' => $this->l("Clé d'accès"),
						'hint' => $this->l("Cette clé d'accès est à reporter dans la configuration de votre site dans Polaris. Elle permet d'établir la liaison entre les deux systèmes de manière fiable et sécurisée."),
						'name' => $baseCfg.'ACCESSKEY',	
						'id' => 'code_'.$cfg_index,
						'size' => 128,
						'required' => true,
						'button' => array(
							'label' => $this->l('Générer une nouvelle clé'),
							'attributes' => array(
								'onclick' => 'VEGAGenKey(\'code_'.$cfg_index.'\', 32)'
								)
							)
					),
					array(
						'type' => 'switch',
						'label' => $this->l("Accès actif"),
						'hint' => $this->l("Utilisez cette option pour activer ou désactiver l'accès sans pour autant perdre votre clé."),
						'name' => $baseCfg.'ACCESSOK',
						'class' => 't',
						'is_bool' => true,
						'values' => array(
										array('id' => 'active_on', 'value' => 1, 'label' => $this->l('Activé')),
										array('id' => 'active_off', 'value' => 0, 'label' => $this->l('Désactivé')),
									)
					),
					array(
						'type' => 'html',
						'name' => 'html_lbl',
						'html_content' => "<hr><h4>Options générales</h4>"
					),
					array(
						'type' => 'select',
						'label' => $this->l("Langue"),
						'name' => $baseCfg.'LANG_ID',
						'required' => true,
						'options' => array(
										'query' => Language::getLanguages(),
										'id' => 'id_lang',
										'name' => 'name',
										)
					),
					array(
						'type' => 'select',
						'label' => $this->l("Devise gérée"),
						'name' => $baseCfg.'CURRENCY_ID',
						'required' => true,
						'options' => array(
										'query' => Currency::getCurrencies(),
										'id' => 'id_currency',
										'name' => 'name',
										)
					),
					array(
						'type' => 'select',
						'label' => $this->l("Gérer les produits du site"),
						'name' => $baseCfg.'CATALOG_SHOP_ID',
						"default" => "",
						'options' => array(
										'query' => $shops,
										'id' => 'id_shop',
										'name' => 'name',
										),
						'required' => true,
					),
					array(
						'type' => 'select',
						'label' => $this->l("Gérer les prix du site"),
						'name' => $baseCfg.'PRICES_SHOP_ID',
						"default" => "",
						'options' => array(
										'query' => $shops,
										'id' => 'id_shop',
										'name' => 'name',
										),
						'required' => true,
					),
					array(
						'type' => 'select',
						'label' => $this->l("Groupe de clients"),
						'hint' => $this->l("Groupe de clients dans lequel enregistrer les clients provenant de l'importation depuis Polaris"),
						'name' => $baseCfg.'CUSTOMERSGROUP_ID',
						'required' => true,
						'options' => array(
										'query' => Group::getGroups($id_lang),
										'id' => 'id_group',
										'name' => 'name',
										)
					),
					array(
						'type' => 'select',
						'label' => $this->l("Fournisseur associé"),
						'hint' => $this->l("Tous les produits associés à ce fournisseur seront gérés par la passerelle."),
						'name' => $baseCfg.'SUPPLIER_ID',
						'required' => true,
						'options' => array(
										'query' => $suppliers,
										'id' => 'id_supplier',
										'name' => 'name',
										)
					),
					array(
						'type' => 'select',
						'label' => $this->l("Fournisseur de récupération"),
						'hint' => $this->l("Tous les produits associés à ce fournisseur seront gérés par la passerelle, mais seulement leur stock et leur prix (achat et vente)."),
						'name' => $baseCfg.'BKSUPPLIER_ID',
						'required' => true,
						'options' => array(
										'query' => $suppliers,
										'id' => 'id_supplier',
										'name' => 'name',
										)
					),
				),	
		
				'buttons' => array(
					array(
						'title' => "Supprimer",
						'class' => 'btn-danger pull-left',
						'icon' => 'process-icon-delete',
						'js' => "if (confirm('Êtes-vous sûr de vouloir supprimer le lien ".$cfg->LinkName." ?')) { $('#".$baseCfg."DELETE').val(1); $('#configuration_form').submit(); } "
					),
					array(
						'title' => "Tri des tailles",
						'class' => 'btn pull-right',
						'icon' => 'process-icon-expand',
						'js' => "if (confirm('Êtes-vous sûr de vouloir retrier les tailles du site ".$cfg->LinkName." ?')) { window.location='".$this->context->link->getAdminLink('AdminPolarisLinkUtils')."&id_link=".$cfg->LinkID."&tool=sort-sizes' } "
					),
					array(
						'title' => $cfg->Resync > 0 ? "Resynchronisation demandée" : "Resynchroniser",
						'class' => 'btn pull-right'.($cfg->Resync > 0 ? " btn-warning" : ""),
						'icon' => 'process-icon-refresh',
						'js' => "window.location='".$this->context->link->getAdminLink('AdminPolarisLinkUtils')."&id_link=".$cfg->LinkID."&tool=resync'"
					),
				
				)
			);	
			
			$fields_form[$cfg_index]['form']['input'][] = array(
						'type' => 'radio',
						'label' => $this->l("Méthode de traitement des thèmes Polaris"),
						'hint' => $this->l("Précise la méthode de gestion des multi-thèmes Polaris"),
						'name' => $baseCfg.'TAGS_METHOD',
						"default" => "",
						"values" => array(
										array("id" => 0, "value" => 0, "label" => "Ignorer les thèmes"),
										array("id" => 1, "value" => 1, "label" => "Considérer comme des catégories"),
										array("id" => 2, "value" => 2, "label" => "Considérer comme des caractéristiques"),
										array("id" => 3, "value" => 3, "label" => "Considérer comme des mots-clés"),
									),
						'required' => true,
					);

			$fields_form[$cfg_index]['form']['input'][] = array(
				'type' => 'html',
				'name' => 'html_lbl',
				'html_content' => "<hr><h4>Classement des références</h4>"
			);

			$fields_form[$cfg_index]['form']['input'][] = array(
						'type' => 'switch',
						'label' => $this->l("Classement automatique"),
						'hint' => $this->l("Utilisez cette option pour classer votre produit selon un type de catégorie de Polaris."),
						'name' => $baseCfg.'AUTOCATEGORY',
						'class' => 't',
						'is_bool' => true,
						'values' => array(
										array('id' => 'active_on', 'value' => 1, 'label' => $this->l('Activé')),
										array('id' => 'active_off', 'value' => 0, 'label' => $this->l('Désactivé')),
									)
					);	
					
			$fields_form[$cfg_index]['form']['input'][] = array(
						'type' => 'select',
						'label' => $this->l("Catégorie de classement"),
						'desc' => $this->l("Précise dans quelle catégorie de Prestashop sont rangés les produits à classer quand vous n'utilisez pas le classement automatique."),
						'name' => $baseCfg.'SORTINGCATEGORYID',
						"default" => "",
						"options" => $list_categories,
						'required' => true,
					);	
					
			$fields_form[$cfg_index]['form']['input'][] = array(
						'type' => 'select',
						'label' => $this->l("Racine de la catégorie par défaut"),
						'desc' => $this->l("Précise dans quelle catégorie racine de Prestashop est sélectionnée la catégorie par défaut du produit. Si vous sélectionnez la racine, la catégorie par défaut ne sera pas modifiée."),
						'name' => $baseCfg.'DEFAULT',
						"default" => "",
						"options" => $list_categories,
						'required' => true,
					);					
			
			$fields_form[$cfg_index]['form']['input'][] = array(
						'type' => 'select',
						'label' => $this->l("Catégorie racine pour les RFS"),
						'desc' => $this->l("Précise dans quelle catégorie racine de Prestashop ranger les RFS de Polaris"),
						'name' => $baseCfg.'RFS',
						"default" => "",
						"options" => $list_categories,
						'required' => true,
					);
					
			$fields_form[$cfg_index]['form']['input'][] = array(
					'type' => 'select',
					'label' => $this->l("Catégorie racine pour les saisons"),
					'desc' => $this->l("Précise dans quelle catégorie racine de Prestashop ranger les saisons de Polaris"),
					'name' => $baseCfg.'SEASON',
					"default" => "",
					"options" => $list_categories,
					'required' => true,
				);
					
			$fields_form[$cfg_index]['form']['input'][] = array(
					'type' => 'select',
					'label' => $this->l("Catégorie racine pour les collections"),
					'desc' => $this->l("Précise dans quelle catégorie racine de Prestashop ranger les collections de Polaris"),
					'name' => $baseCfg.'COLLECTION',
					"default" => "",
					"options" => $list_categories,
					'required' => true,
				);
						
			for ($i = 1; $i <= 4; $i++)
			{
				$fields_form[$cfg_index]['form']['input'][] = array(
						'type' => 'select',
						'label' => $this->l("Catégorie racine pour la catégorie n°" . $i),
						'desc' => $this->l("Précise dans quelle catégorie racine de Prestashop ranger les catégories de niveau ".$i." sous Polaris"),
						'name' => $baseCfg.'CATEGORY'.$i,
						"default" => "",
						"options" => $list_categories,
						'required' => true,
					);
			}
						
			$fields_form[$cfg_index]['form']['input'][] = array(
						'type' => 'select',
						'label' => $this->l("Catégorie racine pour les mots-clés"),
						'desc' => $this->l("Précise dans quelle catégorie racine de Prestashop ranger les mots clés de Polaris lorsqu'ils sont utilisés en mode catégorie"),
						'name' => $baseCfg.'TAGS',
						"default" => "",
						"options" => $list_categories,
						'required' => true,
					);

			$fields_form[$cfg_index]['form']['input'][] = array(
				'type' => 'html',
				'name' => 'html_lbl',
				'html_content' => "<hr><h4>Configuration des tailles</h4>"
			);
			
			$fields_form[$cfg_index]['form']['input'][] = array(
						'type' => 'select',
						'label' => $this->l("Groupe d'attributs pour la taille"),
						'desc' => $this->l("Précise dans quel groupe d'attributs les déclinaisons de taille doivent être rangées"),					
						'name' => $baseCfg.'ATTR_FOR_LV0',
						"default" => "",
						"options" => $list_attrs,
						'required' => true
					);

			$fields_form[$cfg_index]['form']['input'][] = array(
						'type' => 'switch',
						'label' => $this->l("Gérer les grilles de tailles"),
						'hint' => "Quand cette option est activée et si un groupe d'attributs du même nom que la grille de tailles Polaris existe sous Prestashop, la passerelle gérera la taille dans le groupe d\'attributs correspondant; dans le cas contraire, les tailles seront gérées dans le groupe d'attributs précisé ci-dessus.",
						'name' => $baseCfg."SIZE_GRID",
						'class' => 't', 'is_bool' => true,
						'values' => array(
										array('id' => 'active_on', 'value' => 1),
										array('id' => 'active_off', 'value' => 0, 'label' => $this->l('Désactivé')),
									)
						);
										
			for ($i = 1; $i <= 4; $i++)
			{
				$fields_form[$cfg_index]['form']['input'][] = array(
					'type' => 'html',
					'name' => 'html_lbl',
					'html_content' => '<hr><h4>Configuration du niveau Polaris n°'.$i.'</h4>'
				);

				$fields_form[$cfg_index]['form']['input'][] = array(
						'type' => 'select',
						'label' => $this->l("Groupe d'attributs"),
						'desc' => $this->l("Précise dans quel groupe d'attributs les déclinaisons de type niveau ".$i." sous Polaris doivent être gérée sur Prestashop"),					
						'name' => $baseCfg.'ATTR_FOR_LV'.$i,
						"default" => "",
						"options" => $list_nullable_attrs,
						'required' => false,
					);

				$fields_form[$cfg_index]['form']['input'][] = array(
						'type' => 'select',
						'label' => $this->l("Caractéristique"),
						'desc' => $this->l("Précise dans quel groupe d'attributs les déclinaisons de type niveau générique ".$i." sous Polaris doivent être gérée sur Prestashop"),					
						'name' => $baseCfg.'FEATURE_FOR_LV'.$i,
						"default" => "",
						"options" => $list_nullable_features,
						'required' => false,
					);

				$fields_form[$cfg_index]['form']['input'][] = array(
					'type' => 'select',
					'label' => $this->l("Groupe d'attributs pour le niveau générique"),
					'desc' => $this->l("Précise dans quel groupe d'attributs les déclinaisons de type niveau générique ".$i." sous Polaris doivent être gérée sur Prestashop"),					
					'name' => $baseCfg.'ATTR_FOR_LVG'.$i,
					"default" => "",
					"options" => $list_nullable_attrs,
					'required' => false,
				);				

				$fields_form[$cfg_index]['form']['input'][] = array(
					'type' => 'select',
					'label' => $this->l("Caractéristique pour le niveau générique"),
					'desc' => $this->l("Précise dans quel groupe d'attributs les déclinaisons de type niveau générique ".$i." sous Polaris doivent être gérée sur Prestashop"),					
					'name' => $baseCfg.'FEATURE_FOR_LVG'.$i,
					"default" => "",
					"options" => $list_nullable_features,
					'required' => false,
				);
			}
			
			$fields_form[$cfg_index]['form']['input'][] = array(
				'type' => 'html',
				'name' => 'html_lbl',
				'html_content' => "<hr><h4>Options d'import/export</h4>"
			);
					
			$fields_form[$cfg_index]['form']['input'][] = array(
						'type' => 'checkbox',
						'label' => $this->l("Exporter les commandes ayant obtenu le statut"),
						'name' => $baseCfg.'EXPORT_SALES_STATUSES',
						"default" => "",
						'values' => array(
										'query' => OrderState::getOrderStates($id_lang),
										'id' => 'id_order_state',
										'name' => 'name',
										),
						'required' => true,
					);
						
			foreach ($opts as $opt_name => $opt)
			{
				$fields_form[$cfg_index]['form']['input'][] = array(
						'type' => 'switch',
						'label' => $this->l($opt['t']),
						'hint' => $this->l($opt['d']),
						'name' => $baseCfg.''.$opt_name,
						'class' => 't',
						'is_bool' => true,
						'values' => array(
										array('id' => 'active_on', 'value' => 1),
										array('id' => 'active_off', 'value' => 0, 'label' => $this->l('Désactivé')),
									)
					);
			}			
			
			// Load current value
			if (!Configuration::hasKey("POLARISLINK_ALLOW_MAINTENANCE_ACCESS"))
				Configuration::updateValue("POLARISLINK_ALLOW_MAINTENANCE_ACCESS", 1);
			$helper->fields_value["POLARISLINK_ALLOW_MAINTENANCE_ACCESS"] = Configuration::get("POLARISLINK_ALLOW_MAINTENANCE_ACCESS");			

			if (!Configuration::hasKey("POLARISLINK_CRON_DATAUPDATE"))
				Configuration::updateValue("POLARISLINK_CRON_DATAUPDATE", 0);

			$helper->fields_value["POLARISLINK_CRON_DATAUPDATE"] = Configuration::get("POLARISLINK_CRON_DATAUPDATE");			

			$helper->fields_value["POLARISLINK_INSTALL_DATE"] = Configuration::get("POLARISLINK_INSTALL_DATE");			

			$helper->fields_value["POLARISLINK_PRODUCTS_SELECTOR_USE"] = Configuration::get("POLARISLINK_PRODUCTS_SELECTOR_USE");
			$helper->fields_value["POLARISLINK_PRODUCTS_SELECTOR_STOCK"] = Configuration::get("POLARISLINK_PRODUCTS_SELECTOR_STOCK");
			$helper->fields_value["POLARISLINK_PRODUCTS_SELECTOR_EXCLUDE_CURRENT"] = Configuration::get("POLARISLINK_PRODUCTS_SELECTOR_EXCLUDE_CURRENT");
			$helper->fields_value["POLARISLINK_PRODUCTS_SELECTOR_CSS"] = Configuration::get("POLARISLINK_PRODUCTS_SELECTOR_CSS");			
			$helper->fields_value["POLARISLINK_PRODUCTS_SELECTOR_CANONICAL"] = Configuration::get("POLARISLINK_PRODUCTS_SELECTOR_CANONICAL");						

			$helper->fields_value[$baseCfg.'DELETE'] = "0";
			$helper->fields_value[$baseCfg.'LINKID'] = $cfg->LinkID;
			$helper->fields_value[$baseCfg.'ACCESSNAME'] = $cfg->LinkName;
			$helper->fields_value[$baseCfg.'ACCESSKEY'] = $cfg->AccessKey;
			$helper->fields_value[$baseCfg.'ACCESSOK'] = $cfg->AccessActive;
			
			$helper->fields_value[$baseCfg.'LANG_ID'] = $cfg->LangID;
			$helper->fields_value[$baseCfg.'CURRENCY_ID'] = $cfg->CurrencyID;
			
			$helper->fields_value[$baseCfg.'CATALOG_SHOP_ID'] = $cfg->CatalogShopID;
			$helper->fields_value[$baseCfg.'PRICES_SHOP_ID'] = $cfg->PricesShopID;
			
			$helper->fields_value[$baseCfg.'CUSTOMERSGROUP_ID'] = $cfg->CustomersGroupID;
			$helper->fields_value[$baseCfg.'SUPPLIER_ID'] = $cfg->SupplierID;
			$helper->fields_value[$baseCfg.'BKSUPPLIER_ID'] = $cfg->SupplierBackupID;
			$helper->fields_value[$baseCfg.'TAGS_METHOD'] = $cfg->MethodTags;
									
			$helper->fields_value[$baseCfg.'RFS'] = $cfg->CategoryForRFS;
			$helper->fields_value[$baseCfg.'COLLECTION'] = $cfg->CategoryForCollection;
			$helper->fields_value[$baseCfg.'SEASON'] = $cfg->CategoryForSeason;		
			$helper->fields_value[$baseCfg.'TAGS'] = $cfg->CategoryForTags;
			$helper->fields_value[$baseCfg.'CATEGORY1'] = $cfg->CategoryForCategory1;
			$helper->fields_value[$baseCfg.'CATEGORY2'] = $cfg->CategoryForCategory2;
			$helper->fields_value[$baseCfg.'CATEGORY3'] = $cfg->CategoryForCategory3;
			$helper->fields_value[$baseCfg.'CATEGORY4'] = $cfg->CategoryForCategory4;
						
			$helper->fields_value[$baseCfg.'AUTOCATEGORY'] = $cfg->AutoCategory;			
			$helper->fields_value[$baseCfg.'SORTINGCATEGORYID'] = $cfg->SortingCategoryID;			
			$helper->fields_value[$baseCfg.'DEFAULT'] = $cfg->DefaultCategoryRoot;			
			
			$helper->fields_value[$baseCfg.'ATTR_FOR_LV0'] = $cfg->AttrGroupForAttr0;
			$helper->fields_value[$baseCfg.'SIZE_GRID'] = $cfg->OPT_SIZE_GRID;

			$helper->fields_value[$baseCfg.'ATTR_FOR_LV1'] = $cfg->AttrGroupForAttr1;
			$helper->fields_value[$baseCfg.'ATTR_FOR_LV2'] = $cfg->AttrGroupForAttr2;
			$helper->fields_value[$baseCfg.'ATTR_FOR_LV3'] = $cfg->AttrGroupForAttr3;
			$helper->fields_value[$baseCfg.'ATTR_FOR_LV4'] = $cfg->AttrGroupForAttr4;
			$helper->fields_value[$baseCfg.'ATTR_FOR_LVG1'] = $cfg->AttrGroupForAttrG1;
			$helper->fields_value[$baseCfg.'ATTR_FOR_LVG2'] = $cfg->AttrGroupForAttrG2;
			$helper->fields_value[$baseCfg.'ATTR_FOR_LVG3'] = $cfg->AttrGroupForAttrG3;
			$helper->fields_value[$baseCfg.'ATTR_FOR_LVG4'] = $cfg->AttrGroupForAttrG4;

			$helper->fields_value[$baseCfg.'FEATURE_FOR_LV1'] = $cfg->FeatureForAttr1;
			$helper->fields_value[$baseCfg.'FEATURE_FOR_LV2'] = $cfg->FeatureForAttr2;
			$helper->fields_value[$baseCfg.'FEATURE_FOR_LV3'] = $cfg->FeatureForAttr3;
			$helper->fields_value[$baseCfg.'FEATURE_FOR_LV4'] = $cfg->FeatureForAttr4;
			$helper->fields_value[$baseCfg.'FEATURE_FOR_LVG1'] = $cfg->FeatureForAttrG1;
			$helper->fields_value[$baseCfg.'FEATURE_FOR_LVG2'] = $cfg->FeatureForAttrG2;
			$helper->fields_value[$baseCfg.'FEATURE_FOR_LVG3'] = $cfg->FeatureForAttrG3;
			$helper->fields_value[$baseCfg.'FEATURE_FOR_LVG4'] = $cfg->FeatureForAttrG4;
			
			foreach ($opts as $opt_name => $opt)
				$helper->fields_value[$baseCfg.''.$opt_name] = $cfg->$opt_name;
		 
			$values = $cfg->OrderExportStateID;
			if ($values != null && count($values) > 0)
			{
				foreach ($values as $state)
					$helper->fields_value[$baseCfg.'EXPORT_SALES_STATUSES_'.$state] = 1;
			}
		 
		}

		// Configuration du sélecteur de produits connexes
		$special_tabs = [];
		$cfg_index++;
			
		// Init Fields form array		
		$fields_form[$cfg_index]['form'] = array(
			'input' => array(
				array(
					'type' => 'html',
					'name' => 'htmllbl',
					'html_content' => '<hr/><h4>Sélecteur de produits connexes</h4>'.												
										'<p>Si vous utilisez le sélecteur, n\'oubliez pas de paramétrer le format d\'image <code>selection</code> pour les produits, avec une taille par défaut de 48px × 48px.</p>'.
										'<p>Le sélecteur sera injecté sous le prix du produit. Si vous désirez le placer ailleurs, désactivez le sélecteur et utilisez le hook suivant dans votre template : <code>{hook h="actionPolarisLinkProductSelector" product_id=$product->id}</code></p>',
				),
				array(
					'type' => 'switch',
					'label' => $this->l("Sélecteur de produits connexes sur les pages produits"),
					'name' => "POLARISLINK_PRODUCTS_SELECTOR_USE",
					'class' => 't', 'is_bool' => true,
					'values' => array(
									array('id' => 'active_on', 'value' => 1),
									array('id' => 'active_off', 'value' => 0, 'label' => $this->l('Désactivé')),
								)
				),						
				array(
					'type' => 'switch',
					'label' => $this->l("Cacher les déclinaisons sans stock"),
					'name' => "POLARISLINK_PRODUCTS_SELECTOR_STOCK",
					'class' => 't', 'is_bool' => true,
					'values' => array(
									array('id' => 'active_on', 'value' => 1),
									array('id' => 'active_off', 'value' => 0, 'label' => $this->l('Désactivé')),
								)
				),
				array(
					'type' => 'switch',
					'label' => $this->l("Cacher la déclinaison actuellement affichée"),
					'name' => "POLARISLINK_PRODUCTS_SELECTOR_EXCLUDE_CURRENT",
					'class' => 't', 'is_bool' => true,
					'values' => array(
									array('id' => 'active_on', 'value' => 1),
									array('id' => 'active_off', 'value' => 0, 'label' => $this->l('Désactivé')),
								)
				),
				array(
					'type' => 'switch',
					'label' => $this->l("Gérer les liens canoniques des variantes automatiquement"),
					'hint' => 'Protection contre le <i>near-duplicate content (panda)',
					'desc' => "Si vous utilisez la même description pour toutes les variantes (couleur notamment) d'un même produit, il est possible que Google détecte un contenu presque-dupliqué et que votre ".
								"référencement soit pénalisé. En activant cette option, PolarisLink génèrera – s'il le juge utile – le lien canonique dans l'entête HTTP pour signifier à Google que l'indexation de la page variante n'est pas nécessaire.",
					'name' => "POLARISLINK_PRODUCTS_SELECTOR_CANONICAL",
					'class' => 't', 'is_bool' => true,
					'values' => array(
									array('id' => 'active_on', 'value' => 1),
									array('id' => 'active_off', 'value' => 0, 'label' => $this->l('Désactivé')),
								)
				),
				array(
					'type' => 'textarea',
					'label' => $this->l("Style d'affichage du sélecteur"),
					'name' => "POLARISLINK_PRODUCTS_SELECTOR_CSS",							
				),
			));
		$special_tabs[] = [ "idx" => $cfg_index, "title" => "Sélecteur produits" ];
		
		$generatedForm = $helper->generateForm($fields_form);
		
		// Change fieldset to pane
		$panePage = '<ul class="nav nav-tabs">';
		
		$cfg_index = 0;
		foreach ($this->Config as $cfg)
		{
			$cfg_index++;
			$tabPageName = "fieldset_".$cfg_index."_".$cfg_index;
			
			if ($cfg_index == 1)
				$generatedForm = str_replace('class="panel" id="'.$tabPageName.'"', 'class="panel tab-pane fade in active" id="'.$tabPageName.'"', $generatedForm);	
			else
				$generatedForm = str_replace('class="panel" id="'.$tabPageName.'"', 'class="panel tab-pane fade" id="'.$tabPageName.'"', $generatedForm);
			
			$panePage .= '<li class="'.(($cfg_index == 1) ? 'active' : '').'"><a data-toggle="tab" href="#'.$tabPageName.'">'.$cfg->LinkName.'</a></li>';
		}

		foreach ($special_tabs as $cfg)
		{
			$cfg_index = $cfg["idx"];
			$tabPageName = "fieldset_".$cfg_index."_".$cfg_index;
			
			if ($cfg_index == 1)
				$generatedForm = str_replace('class="panel" id="'.$tabPageName.'"', 'class="panel tab-pane fade in active" id="'.$tabPageName.'"', $generatedForm);	
			else
				$generatedForm = str_replace('class="panel" id="'.$tabPageName.'"', 'class="panel tab-pane fade" id="'.$tabPageName.'"', $generatedForm);
			
			$panePage .= '<li class="'.(($cfg_index == 1) ? 'active' : '').'"><a data-toggle="tab" href="#'.$tabPageName.'">'.$cfg["title"].'</a></li>';
		}
		
		$panePage .= '</ul>';		
		$panePage .= '<div class="tab-content">';
		
		//var_dump($generatedForm);
		$generatedForm = str_replace('<div class="panel tab-pane fade in active" id="fieldset_1_1">', $panePage . '<div class="panel tab-pane fade in active" id="fieldset_1_1">', $generatedForm);
		$generatedForm = str_replace('</form>', '</div></form>', $generatedForm);
		
		$generatedForm = str_replace('class="btn btn-default btn-danger', 'class="btn btn-danger', $generatedForm);
		
		$configPage .= $generatedForm;
		
		$this->sendMetrics();
		
		return $output.$configPage;
	}

	public function loadRelatedProductsInfo($products_id)
	{
		$db = Db::getInstance(_PS_USE_SQL_SLAVE_);

		$products = array();
		$productsDesc = array();
		$productsAttrs = array();
		
		if (count($products_id) > 0)
		{
			foreach ($products_id as $id)
				$products[$id] = array();

			$group_per_product = array();
			$products_per_group = array();
			$cacheProducts = array();
						
			foreach ($db->executeS("SELECT DISTINCT id_product, id_group FROM "._DB_PREFIX_."polarislink_products_group WHERE id_group IN (SELECT id_group FROM "._DB_PREFIX_."polarislink_products_group WHERE id_product IN (".join(",", $products_id)."))") as $row)
			{
				if (!isset($cacheProducts[$row["id_product"]]))
					$cacheProducts[$row["id_product"]] = new Product($row["id_product"], true, $this->context->language->id, $this->context->shop->id);
				
				if (!isset($products_per_group[$row["id_group"]]))
					$products_per_group[$row["id_group"]] = array();
								
				$products_per_group[$row["id_group"]][] = $row["id_product"];
				$group_per_product[$row["id_product"]] = $row["id_group"];
			}
			
			foreach ($products_id as $k)
			{
				$group = $group_per_product[$k];
				foreach ($products_per_group[$group_per_product[$k]] as $id_product)
					$products[$k][] = $cacheProducts[$id_product];
			}
						
			foreach ($db->executeS("SELECT DISTINCT pa.id_product, a.id_attribute_group, a.id_attribute, la.name, a.color, p.active ".
									"FROM "._DB_PREFIX_."product_attribute AS pa ".
									"LEFT JOIN "._DB_PREFIX_."product AS p ON (p.id_product = pa.id_product) ".
									"LEFT JOIN "._DB_PREFIX_."product_attribute_combination AS pac ON (pac.id_product_attribute = pa.id_product_attribute) ".
									"LEFT JOIN "._DB_PREFIX_."attribute AS a ON (a.id_attribute = pac.id_attribute) ".
									"LEFT JOIN "._DB_PREFIX_."attribute_group AS ag ON (ag.id_attribute_group = a.id_attribute_group) ".
									"LEFT JOIN "._DB_PREFIX_."attribute_lang AS la ON (la.id_attribute = a.id_attribute AND la.id_lang = ".$this->context->language->id.") ".
									"WHERE pa.id_product IN (".join(",", array_keys($cacheProducts)).")".
									"ORDER BY ag.position, a.position, pa.id_product") as $row)
			{
				$id_product = $row["id_product"];

				if (!isset($productsDesc[$id_product]))
					$productsDesc[$id_product] = array();
				
				if (!isset($productsDesc[$id_product][$row["id_attribute_group"]]))
					$productsDesc[$id_product][$row["id_attribute_group"]] = array();
				
				if (!isset($productsDesc[$id_product][$row["id_attribute_group"]][$row["id_attribute"]]))
					$productsDesc[$id_product][$row["id_attribute_group"]][$row["id_attribute"]] = array(
																										"id_attribute" => $row["id_attribute"],
																										"name" => $row["name"],
																										"color" => $row["color"],
																									);
				
				if ($row["active"])
				{
					$linkUrl = $cacheProducts[$id_product]->getLink();
					foreach ($products_per_group[$group_per_product[$id_product]] as $id_product2)
					{
						if (!isset($productsAttrs[$id_product2]))
							$productsAttrs[$id_product2] = array();
					
						if (!isset($productsAttrs[$id_product2][$row["id_attribute_group"]]))
							$productsAttrs[$id_product2][$row["id_attribute_group"]] = array();

						if (!isset($productsAttrs[$id_product2][$row["id_attribute_group"]][$row["id_attribute"]]))
							$productsAttrs[$id_product2][$row["id_attribute_group"]][$row["id_attribute"]] = array(
																						"linkUrl" => $linkUrl,
																						"id_attribute" => $row["id_attribute"],
																						"name" => $row["name"],
																						"color" => $row["color"],
																					);
					}
				}
			}
		}

		return array(
			"products" => $products,
			"productsDesc" => $productsDesc,
			"productsAttrs" => $productsAttrs,
		);
	}
	
	//
	// Charge les produits connexes
	//
	public function hookActionPolarisLinkLoadRelatedProducts($params)
	{
		$single_id_product = 0;
		
		if (!isset($params["multiple_mode"]) || $params["multiple_mode"] == 0)
		{
			if (isset($params["id_product"]))
			{
				// Mode paramétré
				$single_id_product = intval($params["id_product"]);
			}
			else 
			{
				// Mode détection automatique
				$tplProduct = $params["smarty"]->getTemplateVars("product");
				if ($tplProduct != null)
				{
					if (is_array($tplProduct))
					{
						if (isset($tplProduct["id_product"]))
							$single_id_product = intval($tplProduct["id_product"]);
					}
					else
						$single_id_product = intval($tplProduct->id);
				}
			}
		}
		
		$products = array();
		if ($single_id_product > 0)
		{
			$products[$single_id_product] = array();
		}
		else
		{		
			// Toujours là ?
			if (isset($params["id_products"]))
			{
				// Mode paramétré
				if (is_array($params["id_products"]))
				{
					foreach ($params["id_products"] as $id)
						$product[$id] = array();
				}
				else
				{
					$products[$params["id_products"]] = array();
				}
			}
			else
			{
				// Mode automatique
				$tplProducts = $params["smarty"]->getTemplateVars("products");
				if ($tplProducts != null)
				{
					foreach ($tplProducts as $tplProduct)
						$products[$tplProduct["id_product"]] = array();
				}
			}
		}
				
		$results = $this->loadRelatedProductsInfo(array_keys($products));

		$params["smarty"]->assign("relatedProductsAttributes", 
											array(
												'img_col_dir' => _THEME_COL_DIR_,
												'col_img_dir' => _PS_COL_IMG_DIR_,
												'products' => $results["productsAttrs"],
												'details' => $results["productsDesc"],
												));

		if ($single_id_product > 0)
			$params["smarty"]->assign("relatedProducts", $results["products"][$single_id_product]);
		else
			$params["smarty"]->assign("relatedProducts", $results["products"]);
	}
}
